/**
 * \file gre/3ddc.h 
 * \brief Class for drawing context in 3d
 *
 * \if NODOC
 * $Id: 3ddc.h_v 1.16 2011/10/06 15:20:20 mju Exp $
 *
 * $Log: 3ddc.h_v $
 * Revision 1.16  2011/10/06 15:20:20  mju
 * Remove dependencies.
 *
 * Revision 1.15  2005/07/22 21:48:31  vdronov
 * added bitmap pattern rendering
 *
 * Revision 1.8  2003/09/26 17:18:49  vdronov
 * added triangle drawing for strip and fan mode
 *
 * Revision 1.7  2003/08/20 18:00:52  vdronov
 * changed m_Vertices from FPOINT3DH to DPOINT3DH because of precision loss
 *
 * Revision 1.1  2003/03/26 14:25:32  v2  vdronov
 * Initial revision
 * \endif
**/ 

#ifndef	INC_GRE_3DDC_H
#define	INC_GRE_3DDC_H

#ifndef	INC_GRE_3DUTILS_H
#include <gre/3dutils.h>
#endif

#ifndef	INC_GRE_3DSCENE_H
#include <gre/3dscene.h>
#endif

#ifndef	INC_MI32_FILLPOLY_H
#include <mi32/fillpoly.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

#ifndef	INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace GRE {

//! Provide common methods to draw in 3D view.
class DRAWINGCONTEXT3D {
	public:

		DRAWINGCONTEXT3D ();

		virtual ~DRAWINGCONTEXT3D ();

		//! Initialization of drawing context
		void DrawBegin (
			SCENE3D* scene,
			MGD::CONTEXT* context 
			);

		//! Finalization of drawing context
		void DrawEnd ();

		void DrawPoint (
			const DPOINT3D& point
			);

		void DrawLine (
			const DPOINT3D& begin,
			const DPOINT3D& end
			);

		//! Draw array of triangles using generic structure TRIANGULATION
		void DrawTriangles ( 	
			const TRIANGULATIONVECTOR& triangulations
			);

		//! Draw array of pixels using generic structure PIXALIZATION
		void DrawPixels ( 	
			const PIXALIZATIONVECTOR& pixalizations
			);

		void RenderPixel (
			const INT32 col, 
			const INT32 lin, 
			const double z
			); 

		void SetColor (
			const COLOR& color
			) { if (m_Context != 0) m_Context->SetColor(color); }

		SCENE3D* GetScene (
			) { return m_Scene; }

		double GetPixelSizeInMM (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class SPAN : public FILLPOLYGON::SPAN {
			public:

				SPAN (
					DRAWINGCONTEXT3D *dc3d
					) :
					m_DC3D(dc3d)
					{ }

			private:

				void VProcess (
					const double x1, 
					const double x2, 
					const double y
					);

				SPAN (
					);

				DRAWINGCONTEXT3D *m_DC3D;
			};

		friend class SPAN;

		MGD::CONTEXT *m_Context; 
		SCENE3D *m_Scene;

		TRIANGULATION *m_Triangulation;

		DOUBLE_ARRAY<DPOINT3DH> m_Vertices;
		DOUBLE_ARRAY<DPOINT3DH> m_Points;

		DOUBLE_ARRAY<DPOINT3DH> m_TempPoints;
		POLYLINE m_Polyline;

		SCREENPLANE m_ScreenPlane;

		void FillSpan (const INT32 col1, const INT32 col2, const INT32 lin);
		void FillSolidSpan (const INT32 col1, const INT32 col2, const INT32 lin);
		void FillPointsFieldSpan (const INT32 col1, const INT32 col2, const INT32 lin);

		void DrawTINWireframe (const TRIANGULATION *triangulation);
		void DrawTINSolid (const TRIANGULATION *triangulation);

		void DrawStripWireframe (const TRIANGULATION *triangulation);
		void DrawStripSolid (const TRIANGULATION *triangulation);

		void DrawFanWireframe (const TRIANGULATION *triangulation);
		void DrawFanSolid (const TRIANGULATION *triangulation);

		void DrawTexturizedPixels (const PIXALIZATION *pixalization);
		void DrawColoredPixels (const PIXALIZATION *pixalization);
		void DrawPointsFieldPixels (const PIXALIZATION *pixalization);
		void DrawBitmapPixels (const PIXALIZATION *pixalization);

		#endif  // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of namespace

#endif
