/**
 * \file gre/lrlegend.h
 * \brief GRE::LAYER_LEGEND class definitions.
 *
 * \if NODOC
 * $Id: lrlegend.h_v 1.16 2011/10/20 13:38:15 mju Exp $
 *
 * $Log: lrlegend.h_v $
 * Revision 1.16  2011/10/20 13:38:15  mju
 * Remove unneeded hdr.
 *
 * Revision 1.15  2011/10/20 13:26:07  mju
 * Remove redundant hdrs.
 *
 * Revision 1.14  2007/05/21 13:33:52  mju
 * Use objitem hdr.
 *
 * Revision 1.13  2006/10/24 20:50:58  dwilliss
 * Added v_usestransparency
 *
 * Revision 1.12  2006/09/15 15:30:23  mju
 * Serializer changes.
 *
 * Revision 1.11  2005/12/21 00:08:32  scowan
 * Replaced use of fileNAME_MAX with objitem's.
 *
 * Revision 1.10  2005/10/26 16:36:43  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.9  2005/10/14 17:31:17  mju
 * GRE namespace and treeview.
 *
 * Revision 1.8  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.7  2005/02/28 17:53:24  scowan
 * Support layer copy virtual method.
 *
 * Revision 1.6  2005/01/20 16:56:35  mju
 * Redesign layer type registration.
 *
 * Revision 1.5  2004/10/01 17:19:55  mju
 * Add dlgparent to assignObject.
 *
 * Revision 1.4  2004/03/05 18:53:41  mju
 * Use virtual dtor if virtual fns.
 *
 * Revision 1.3  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/07/30 15:21:53  mju
 * Ignore private section.
 *
 * Revision 1.1  2002/09/20 17:37:28  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRLEGEND_H
#define	INC_GRE_LRLEGEND_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

namespace GRE {

//===================================================================================================================

class LAYER_LEGEND : public GRE::LAYER {
	public:

		enum SUBTYPE {
			//! These types are stored in layouts by number
			//! and thus must not be changed
			SUBTYPE_Point =			0,
			SUBTYPE_Line =				1,
			SUBTYPE_Poly =				2,
			SUBTYPE_ColorSample =	3,
			SUBTYPE_ColorScale =		4,
			SUBTYPE_RasterSample =	5
			};

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				RVC::OBJITEM m_ObjItem;
				RVC::OBJITEM m_ObjItemStyle;

			/*=================This stuff must be saved/loaded by mdlegend.c=============*/
				INT32 flags;								//!< LEGENDFLAG_...
				SUBTYPE LegendType;						//!< Legend subtype
				int StyleTable;							//!< Style assignment table (-1 if not used)
				int NumCols;								//!< # of columns of legend stuff
				TEXTSTYLE Style[4];						//!< Font styles to use
				COLOR	BackgroundColor;					//!< Color for background (default: transparent)
				COLOR SampleBorderColor;				//!< Sample border color
				COLOR LegendBorderColor;				//!< Legend border color
				//! All measurements are stored in output mm
				double SampleBorderWidth;
				double LegendBorderWidth;
				double ColSpace;							//!< Space between columns
				double SampleSpace;						//!< Space between sample and description
				double SampleHeight;						//!< Size of sample
				double SampleWidth;						//!< Size of sample
				double SampleOffset;						//!< Offset from left edge
				double MapScale;							//!< Map scale all measurements are relative to
				double LineSpace;							//!< Minimum space between lines
				double MarginWidth;						//!< Margin around whole legend
				int units;									//!< Units user wants to edit sizes in

				//! Stuff for LAYERTYPE_LegendColorScale
				double MinIndex;							//!< Minimum colormap index
				double MaxIndex;							//!< Maximum colormap index
				UINT32 NumTicksL;							//!< Number of ticks
				UINT32 NumTicksR;							//!< Number of ticks
				double MinValL;							//!< Minimum Value
				double MinValR;							//!< Minimum Value
				double MaxValL;							//!< Maximum Value
				double MaxValR;							//!< Maximum Value

			/*=================This stuff is just internal=========================*/

				double Height, Width;		//!< Size of layer in ground meters (computed)
				double FullDescHeightM;	

				DISPPARM ();
				DISPPARM (const DISPPARM& rhs) {
					Copy(rhs);
					}
				virtual ~DISPPARM () {
					Free();
					}

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM&);
				void Free ();
				#endif //!< GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType (
			);

		//! Construct layer and insert into list
		explicit LAYER_LEGEND (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None		//!< Creation flags
			);

		virtual ~LAYER_LEGEND ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
	#if defined(X_NATIVE)
		static void CB_CreateDialog (Widget, void*, void*);
	#endif
	#ifdef USE_MGUI
		static void OnMenuCreateDialog (MGUI::ID subtype, bool);
	#endif
		#endif // GENERATING_DOXYGEN_OUTPUT

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		int DrawLegend (void*, int);
		int SetupLegend (void*);

		//! Overrides from GRE::LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_TemplateRead (SERIALIZER& serializer);
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE*) const;

		LAYER_LEGEND (const LAYER_LEGEND&);
		LAYER_LEGEND& operator= (const LAYER_LEGEND&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#define LEGENDFLAG_ConstantSize		0x00000001
#define LEGENDFLAG_RowMajor 	   	0x00000002
#define LEGENDFLAG_LayoutScale   	0x00000004	//!< Use layout scale for sizes
#define LEGENDFLAG_RightJustify  	0x00000008	//!< Right justify descriptions
#define LEGENDFLAG_SampleOnRight 	0x00000010	//!< Samples on the right
#define LEGENDFLAG_SideA				0x00000040  //!< Labels on top/left
#define LEGENDFLAG_SideB				0x00000080  //!< Labels on bottom/right
#define LEGENDFLAG_DescCenterVert	0x00000100
#define LEGENDFLAG_CenterHeadings	0x00000200
#define LEGENDFLAG_Horizontal		LEGENDFLAG_RowMajor

//===================================================================================================================
}	// End namepace GRE

#endif	//!< INC_GRE_LRLEGEND_H
