/** 
 * \file gre/uiformzoom.h 
 * \brief Definitions for GRE::UI::FORM_ZOOM class.
 *
 * \if NODOC
 * $Id: uiformzoom.h_v 1.5 2013/01/30 16:18:39 mju Exp $
 *
 * $Log: uiformzoom.h_v $
 * Revision 1.5  2013/01/30 16:18:39  mju
 * Use form_book so don't have unwanted resize when change mode.
 *
 * Revision 1.4  2011/11/28 17:13:08  mju
 * Fix dox dependency.
 *
 * Revision 1.1  2011/07/11 19:55:01  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIFORMZOOM_H
#define	INC_GRE_UIFORMZOOM_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_FORMBOOK_H
#include <mgui/formbook.h>
#endif

#ifndef INC_SPATMOD_IMAGETILESET_H
#include <spatmod/imagetileset.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
namespace GRE {
	class VIEW;
	class MSG;
	}
#endif

namespace GRE {
namespace UI {

//=====================================================================================================================

//! Form form setting zoom factor on VIEW.
class FORM_ZOOM : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_ZOOM ();

		//! Destructor.
		virtual ~FORM_ZOOM ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			VIEW *view
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_ZOOM THISCLASS;

		enum MODE {
			MODE_Scale = 1,
			MODE_Width,
			MODE_Height,
			MODE_LevelGoogleMaps,
			MODE_LevelBingMaps,
			MODE_LevelGoogleEarth,
			MODE_LevelNASAWorldWind
			};

		enum PAGEID {
			PAGEID_Scale,
			PAGEID_Level,
			PAGEID_Size
			};

		VIEW *m_view;
		UOM::UNIT m_unit;
		SPATMOD::IMAGE::TILESET m_Tileset;
		MGUI::FORM_COMBOBOX m_CbxMode;
		MGUI::FORM_BOOK m_Book;
		MGUI::CTRL_EDIT_NUMBER m_EditScale;
		MGUI::CTRL_PUSHBUTTON m_BtnScaleList;
		MGUI::FORM_EDIT_NUMBER m_EditLevel;
		MGUI::CTRL_PUSHBUTTON m_BtnLevels;
		MGUI::CTRL_EDIT_NUMBER m_EditSize;
		MGUI::CTRL_LABEL m_LblSizeUnits;
		MGUI::CTRL_PUSHBUTTON m_BtnSizeUnits;

		static void GRECB_View (const GRE::MSG *pMsg, void *vpForm);

		double CalcLevelFromZoom (double zoom);
		ERRVALUE CalcViewableSize (DPOINT2D& size);
		void OnBtnLevels ();
		void OnBtnScaleList ();
		void OnBtnSizeUnits ();
		void OnCbxMode ();
		void OnEditLevel ();
		void OnEditScale ();
		void OnEditSize ();
		void OnGreMsgView (const GRE::MSG& msg);
		bool SetupTileset ();
		void UpdateEditSize ();
		void ZoomToLevel (double value);
		void ZoomToHeight (double value);
		void ZoomToWidth (double value);
		void ZoomToScale (double value);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace GRE

#endif		//	INC_GRE_UIFORMZOOM_H
