/**
 * \file gre/view.h
 * \brief GRE VIEW classes
 *
 * \if NODOC
 * $Id: view.h_v 1.206 2013/04/16 15:21:32 mju Exp $
 *
 * $Log: view.h_v $
 * Revision 1.206  2013/04/16 15:21:32  mju
 * Add transDeviceToView.
 *
 * Revision 1.205  2013/02/19 14:53:55  mju
 * Add dlgSnapshotSettings.
 *
 * Revision 1.204  2012/08/09 14:07:12  mju
 * Don't inline cursorTrackDraw/Restore.
 *
 * Revision 1.203  2012/08/08 22:53:34  mju
 * Make v_cursorTrackDraw/Restore virtual and add cursorTrackGet/HasPos.
 *
 * Revision 1.200  2012/03/29 21:46:08  mju
 * Use stereorender class.
 *
 * Revision 1.197  2011/12/01 22:23:00  mju
 * Add getZView.
 *
 * Revision 1.196  2011/11/01 15:13:15  mju
 * Add useStereo to getDevice.
 *
 * Revision 1.195  2011/10/31 17:35:04  mju
 * Add useStereo parm to v_getDrawingDeviceMemory.
 *
 * Revision 1.194  2011/10/28 20:09:46  mju
 * Support dual frame buffers for stereo.
 *
 * Revision 1.191  2011/10/13 17:37:56  mju
 * Add get/setNeedNotifyDrawElement.
 *
 * Revision 1.188  2011/10/05 17:25:41  mju
 * Add notifyChangeViewPoint3D.
 *
 * Revision 1.187  2011/09/29 12:26:17  mju
 * Use stereodevice class.
 *
 * Revision 1.185  2011/03/07 17:38:40  bcolombe
 * Add ReverseGeocode datatip
 *
 * Revision 1.184  2011/02/07 23:21:38  bcolombe
 * Add DrawLayerOverlay
 *
 * Revision 1.182  2010/04/23 16:55:16  mju
 * Use unsignedset.
 *
 * Revision 1.181  2009/08/10 16:43:10  mju
 * Add setZoomMaxDetail and deprecate setZoomOneToOne.
 *
 * Revision 1.178  2008/09/02 17:12:17  mju
 * Add clearQueuedGroupItems.
 *
 * Revision 1.173  2008/07/02 17:04:25  mju
 * Add methods to draw mattes, hasQueuedJobsLocal.
 *
 * Revision 1.171  2008/06/13 21:01:56  mju
 * Add drawGroupMatte and setUseThreads.
 *
 * Revision 1.167  2008/06/02 15:54:25  ldyas
 * adding google locked variables
 *
 * Revision 1.165  2008/05/19 14:07:36  mju
 * Add v_onJobStarted/JobsAllDone.
 *
 * Revision 1.163  2008/05/09 17:43:22  mju
 * Add stuff for remote layer rendering in separate threads.
 *
 * Revision 1.150  2008/02/11 15:56:17  mju
 * Add view parm to delegates.
 *
 * Revision 1.149  2008/02/11 15:27:17  mju
 * Add delegate lists for pointerEnter/Leave/Move.
 *
 * Revision 1.146  2008/01/25 15:34:23  mju
 * Add updateScriptUI and virtual overridable.
 *
 * Revision 1.145  2008/01/18 20:53:58  mju
 * Make getDrawingDevice non-inline.
 *
 * Revision 1.144  2008/01/18 17:43:29  mju
 * Make updateUI public.
 *
 * Revision 1.143  2008/01/17 22:17:40  mju
 * Make getGeorefGroup public.
 *
 * Revision 1.142  2007/12/12 16:35:47  mju
 * Allow saveSnapshotToRaster for use_mgui.
 *
 * Revision 1.141  2007/12/12 16:32:43  mju
 * Allow cursorTrackRestore/SetPosAll for use_mgui.
 *
 * Revision 1.140  2007/12/06 18:41:17  mju
 * Make setPlace non-inline.
 *
 * Revision 1.139  2007/12/06 15:18:51  mju
 * In setNextPosn clear scaleOneToOne flag.
 *
 * Revision 1.138  2007/12/03 18:56:52  mju
 * Add parm to cursorTrackUpdateAll for view to skip.
 *
 * Revision 1.137  2007/12/03 15:20:35  mju
 * Add protected setInteractive method.
 *
 * Revision 1.136  2007/11/29 18:00:13  mju
 * Add createflag_disableRedraw.
 *
 * Revision 1.134  2007/10/15 19:18:54  mju
 * Use dmstype instead of degtostrflags.
 * Add setNextPlace.
 *
 * Revision 1.133  2007/10/15 15:33:33  mju
 * Add getGroup/Layout.
 *
 * Revision 1.132  2007/10/12 15:14:46  mju
 * Move coordsource to view_legacy and rename.
 * Add accessor methods needed by view_canvas.
 *
 * Revision 1.131  2007/10/11 22:50:15  mju
 * Make drawViewable protected instead of private.
 *
 * Revision 1.130  2007/10/11 22:34:48  mju
 * Add drawstate for drawBegin/End.
 *
 * Revision 1.129  2007/10/11 20:00:08  mju
 * Add drawBegin/End.
 *
 * Revision 1.128  2007/10/11 17:46:56  mju
 * Use virtuals for datatip.
 *
 * Revision 1.127  2007/10/05 17:57:57  mju
 * Move more stuff to view_legacy.
 * use viewpos class.
 *
 * Revision 1.124  2007/10/04 19:39:54  mju
 * Don't include viewlegacy unless X and deprecate also.
 *
 * Revision 1.123  2007/10/04 17:55:55  mju
 * Move toollist to view_legacy.
 *
 * Revision 1.117  2007/09/28 13:53:41  mju
 * Add view_legacy.
 *
 * Revision 1.116  2007/07/30 15:39:33  mju
 * Add savesettings methods to set whether to show positions.
 *
 * Revision 1.115  2007/07/05 20:55:50  mju
 * Add cursorTrackRestoreAll/SetPosAll.
 *
 * Revision 1.114  2007/05/24 17:19:53  dwilliss
 * Added optional parm to data tip get string to force HTML output
 *
 * Revision 1.113  2007/05/18 22:54:00  dwilliss
 * added optional parameter to data tip get string
 *
 * Revision 1.110  2007/05/04 21:39:51  mju
 * Add ability for view to have own stereo parameters.
 * Have separate shared stereo parms for 2D vs 3D.
 * Add dialog ptr for 2d stereo settings.
 *
 * Revision 1.109  2007/05/03 21:29:27  mju
 * Add stereo viewing members.
 *
 * Revision 1.108  2007/04/30 19:58:01  mju
 * Add invalidateAllLayers, and do when toggle stereo mode.
 *
 * Revision 1.107  2007/04/27 17:47:35  mju
 * Add member for requesting stereo.
 *
 * Revision 1.106  2007/04/23 20:38:51  dwilliss
 * added static method to get sml class members
 *
 * Revision 1.103  2007/03/30 20:09:42  ldyas
 * added google match option.
 *
 * Revision 1.102  2007/01/16 15:58:54  mju
 * Add getDftColor.
 *
 * Revision 1.101  2006/12/27 17:29:51  mju
 * Add datatipselection_ObjectNames.
 *
 * Revision 1.100  2006/12/11 18:27:06  mju
 * Add showDftTables for highlightElementsByPoint.
 *
 * Revision 1.99  2006/12/06 21:04:22  dwilliss
 * Add a parameter to DataTipGetString
 *
 * Revision 1.97  2006/10/10 13:39:36  mju
 * Use miconstruct* instead of micon.
 *
 * Revision 1.92  2006/06/08 19:29:05  mju
 * Add globaloption snapshot settings for viewer launch and max index.
 *
 * Revision 1.90  2006/04/03 14:10:20  mju
 * Add dataTipSetDelay/PixelDelta.
 *
 * Revision 1.89  2006/03/29 15:58:58  mju
 * Add globaloptions.getSnapshotFolder.
 *
 * Revision 1.88  2006/03/02 17:54:46  mju
 * Add fns for zoombox and recenter tools.
 *
 * Revision 1.87  2006/03/01 18:12:24  mju
 * Add lastmessage so can restore after transient message.
 * Add snapshotIndex to globaloptions.
 *
 * Revision 1.86  2006/02/16 16:15:43  mju
 * Add globaloption for snapshot prefix.
 *
 * Revision 1.85  2006/01/05 21:20:48  mju
 * Add globaloption for autoraise.
 *
 * Revision 1.84  2006/01/05 18:48:33  mju
 * Add more snapshot settings.
 *
 * Revision 1.83  2005/12/21 18:53:23  mju
 * Allow enum bitwise ops for iconview.
 *
 * Revision 1.82  2005/12/21 18:50:25  mju
 * Use enum for iconview, rename method for adding icons.
 *
 * Revision 1.81  2005/12/16 18:48:03  mju
 * Add stuff for handling extent when crs changed.
 *
 * Revision 1.80  2005/12/09 17:46:03  mju
 * Add member for autodraw toggle.
 *
 * Revision 1.79  2005/12/08 22:41:03  mju
 * Add stuff for saving quick snapshot.
 * In globaloptions remove inigroup parm.
 *
 * Revision 1.78  2005/11/28 18:53:51  mju
 * Add viewposn.centerCRS.
 *
 * Revision 1.77  2005/11/18 17:48:36  mju
 * Forward declare smlflag.
 *
 * Revision 1.76  2005/11/18 17:39:59  mju
 * Add createflag_noDftMarkIcons.
 * Add sml_GetCreateFlags.
 *
 * Revision 1.75  2005/11/04 18:53:14  mju
 * enums in gre namespace.
 *
 * Revision 1.72  2005/10/21 20:10:34  scowan
 * Moved static global values from class to static methods to get the items.
 * Attempting to get non-x libraries to be valid again.
 *
 * Revision 1.66  2005/09/20 12:45:51  mju
 * Add datatipselection_autoscale.
 *
 * Revision 1.65  2005/09/14 13:51:48  mju
 * Add zoomToViewableRect.
 *
 * Revision 1.64  2005/07/12 17:45:06  mju
 * Add isRedrawDisabled.
 *
 * Revision 1.61  2005/05/25 21:24:47  mju
 * Add checkRenderBuffersValid.
 *
 * Revision 1.58  2005/05/02 17:44:31  mju
 * Add getTopViewable and highlightMode methods.
 *
 * Revision 1.57  2005/04/29 17:49:55  mju
 * Support rendertarget.
 *
 * Revision 1.56  2005/04/22 21:25:05  mju
 * Make getPixelSizeMM non-inline and add dpoint2d variant.
 *
 * Revision 1.55  2005/04/22 17:35:49  mju
 * Add 'work device' for rendering separate layers.
 *
 * Revision 1.54  2005/04/20 21:20:45  mju
 * Add viewflag_allowPanOffEdge.
 *
 * Revision 1.46  2005/02/18 19:06:51  mju
 * Add static timeLastRefresh.
 * Add device for view-in-view.
 * Separate rendering now partly works.
 *
 * Revision 1.39  2005/02/07 18:51:33  mju
 * Use device_x32 as combined memory/pixmap device.
 * Change restore methods to not restore from memory unless specifically requested
 *   as no longer common (possible, but difficult) to draw directly to pixmap.
 *
 * Revision 1.32  2004/11/29 17:29:06  mju
 * Add detach method and check for null viewable before deref in inlines.
 *
 * Revision 1.1  2003/06/24 16:24:54  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_VIEW_H
#define	INC_GRE_VIEW_H

#ifndef	INC_GRE_OBJECTBASE_H
	#include <gre/objectbase.h>
#endif

#ifndef	INC_GRE_VIEWPOS_H
	#include <gre/viewpos.h>
#endif

#ifndef	INC_GRE_GEOLOCK_H
	#include <gre/geolock.h>
#endif

#ifndef	INC_MI32_STEREOVIEW_H
	#include <mi32/stereoview.h>
#endif

#ifndef	INC_MI32_SERIALIZABLE_H
	#include <mi32/serializable.h>
#endif

#ifndef	INC_MI32_LATLONFM_H
	#include <mi32/latlonfm.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_CRITSEC_H
	#include <mi32/critsec.h>
#endif

#ifndef	INC_MI32_MITHREAD_H
	#include <mi32/mithread.h>
#endif

#if !defined(INC_MGUI_TIMER_H) && defined(USE_MGUI)
	#include <mgui/timer.h>
#endif

#ifndef INC_MAP
#define INC_MAP
#include <map>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT

namespace RVC {
	class ELEMENT;
	}

namespace GRE {
	class DLG_VIEWZOOM;
	class VIEW_LEGACY_STATUSINFO;
	class VIEWCOMMON;
	}

namespace MGD {
	class DEVICE;
	class DEVICE_MEM32RGBA;
	class CONTEXT;
	}

namespace MGUI {
	class CTRL_TOOLTIP;
	}

namespace SPATMOD {
	namespace IMAGE {
		class STAGE_ARRAY;
		}
	}

namespace SML {
	union ARG;
	class FLAG;
	class CLASSMEMBER;
	class CONTEXT;
	}

class UNSIGNEDSET;
class REGION2D;
class RENDERBUFFER;
class STEREODEVICE;
class STEREORENDER;
class MISTATUSMANAGER;
class MISTATUSCONTEXT;
class VIEW_LEGACY_DATATIP;
class ZVIEW;

#endif


enum GRE_PROJACCUR {
	GRE_PROJACCUR_Exact =	2,			//!< Perform exact projection transformation (slowest for all object types)
	GRE_PROJACCUR_Auto =		3,			//!< Do automatic determination between affine and exact
	};

enum VIEWFLAGS {
	VIEWFLAG_None =						0,
	VIEWFLAG_Locator =					0x00000001,	//!< Locator view
	VIEWFLAG_3D =							0x00000002,	//!< View is 3D
	VIEWFLAG_AllowDiffZoom =			0x00000010,	//!< Allow different X/Y zooming
	VIEWFLAG_Hardcopy =					0x00000020,	//!< Hardcopy view
	VIEWFLAG_NoDrawLayers =				0x00000080,	//!< Don't draw any layers in view
	VIEWFLAG_NoDestroyShell =			0x00000100,	//!< Don't destroy Shell containing view when view destroyed
	VIEWFLAG_DialogModal =				0x00000400,	//!< View dialog is modal
	VIEWFLAG_AllowGeoLock =				0x00000800,	//!< Allow GeoLocking
	VIEWFLAG_NoCloseOption =			0x00001000,	//!< Don't include View/Close menu option
	VIEWFLAG_DestroyOnClose =			0x00002000,	//!< Destroy view when user selects View/Close
	VIEWFLAG_NoSetPixmapBG =			0x00004000,	//!< Don't set backgroundPixmap for DrawingArea
	VIEWFLAG_NoSetPrevViewPos =		0x00008000,	//!< Don't set "previous view position" when draw
	VIEWFLAG_DisableDataTips =			0x00010000,	//!< Disable datatips for this view
	VIEWFLAG_NoRenderSeparate =		0x00020000,	//!< Don't render to separate memory devices
	VIEWFLAG_AutoPanGPS =				0x00040000,	//!< Auto-pan view to GPS if available
	VIEWFLAG_NoBlankScreen =			0x00080000,	//!< Don't blank screen while drawing view
	VIEWFLAG_NoEditColorHighlight =	0x00100000,	//!< Don't allow editing of selected/active colors
	VIEWFLAG_NoFillBackground =		0x00200000,	//!< Don't fill background before rendering, used for raster
	VIEWFLAG_AllowPanOffEdge =			0x00400000,	//!< Allow panning off extent of viewable object
	VIEWFLAG_NoShowTrackingCursor =	0x00800000,	//!< Don't display "tracking" cursor in this view
	VIEWFLAG_AlwaysTestCancel =		0x02000000,	//!< Test for cancel from status even if not drawing
	VIEWFLAG_NoDrawOtherViews =		0x04000000,	//!< Don't draw other views when this view is drawn
	VIEWFLAG_NoShowInLocator =			0x08000000,	//!< Don't show this view in locator tool

	VIEWFLAG_SaveMask =					(VIEWFLAG_AllowDiffZoom)
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(VIEWFLAGS);
#endif

//===================================================================================================================
namespace GRE {

class VIEW : public GRE::OBJECT {

	public:

		//! Flags for view creation.
		enum CREATEFLAGS {
			CREATEFLAG_None =							0x00000000,
			CREATEFLAG_3D =							0x00000001,	//!< View is 3D
			CREATEFLAG_NoCallCreateCB =			0x00000002,	//!< Don't call callback list when object created
			CREATEFLAG_NoRenderSeparate =			0x00000004,	//!< Don't render to separate memory buffers
			CREATEFLAG_Hardcopy =					0x00000008,	//!< Create "hardcopy" view
			CREATEFLAG_DisableDraw =				0x00000010,	//!< Start with drawing initially disabled
			CREATEFLAG_Locator =						0x00000020,	//!< Create as "locator" view
			};

		enum COLORUSE {
			COLORUSE_Background,
			COLORUSE_HighlightElem,
			COLORUSE_ActiveElem,
			COLORUSE_DataTipElem,
			COLORUSE_COUNT
			};

		enum DATATIPSELECTION {
			DATATIPSELECTION_FromView =			0,	//!< Use selection set by view
			DATATIPSELECTION_None =					1,	//!< No DataTips
			DATATIPSELECTION_ActiveLayer =		2,	//!< Select for active layer only
			DATATIPSELECTION_AllLayers =			3,	//!< Select for all layers
			DATATIPSELECTION_AllVisible =			4,	//!< Select for all 'visible' layers
			DATATIPSELECTION_AutoScale =			5,	//!< Auto select based on layer scale ranges
			DATATIPSELECTION_ObjectNames =		6,	//!< Group, Layer, File, Object names
			DATATIPSELECTION_ReverseGeocode =	7,	//!< Reverse geocode location information
			};

		//! Delegate type for pointer enter/leave notification.
		typedef fastdelegate::FastDelegate<void(VIEW*)> DELEGATE_POINTERENTERLEAVE;

		//! Delegate type for pointer movement notification.
		typedef fastdelegate::FastDelegate<void(VIEW*, const DPOINT2D&)> DELEGATE_POINTERMOVE;

	#ifdef USE_MGUI
		//! Restore tracking cursor area on all views (X only).
		static void CursorTrackRestoreAll ();
	#endif

	#ifdef USE_MGUI
		//! Set tracking cursor position on all views (X only).
		static void CursorTrackSetPosAll (
			const DPOINT2D& pt,
			const SPATREF::COORDREFSYS& crs,
			const GRE::VIEW *viewskip = 0		//!< View to skip if instigator
			);
	#endif

		//! Edit global settings via dialog.
		//! @return true if settings changed, false if not.
		static bool DlgEditGlobalSettings (
			MDLGPARENT dlgparent							//!< Parent for dialog
			);

		//! Open modal dialog for snapshot settings.
		static ERRVALUE DlgSnapshotSettings (
			MDLGPARENT dlgparent							//!< Parent for dialog
			);

		//! Find next view containing specified group
		static VIEW* FindNextOfGroup (
			const GRE::GROUP *group,			//!< Group to find view of
			VIEW *view = 0							//!< View to start search after, 0 to check first view
			);

		//! Find next view containing specified layer.
		static VIEW* FindNextOfLayer (
			const GRE::LAYER *layer,			//!< Layer to find view of
			VIEW *view = 0							//!< View to start search after, 0 to check first view
			);

		//! Get default color for specified usage.
		static void GetDftColor (
			COLORUSE coloruse,					//!< Color usage
			COLOR& color							//!< Color returned
			);

		//! Get global options which apply to all views
		static const VIEWCOMMON& GetGlobalOptions (
			) { return (GetGlobalOptionsPriv()); }
		
		//! Get shared stereo parameters for 2D.
		static const STEREODEVICE& GetStereoDeviceShared2D (
			) { return (GetStereoDevicePriv2D()); }

		//! Get shared stereo parameters for 3D.
		static const STEREODEVICE& GetStereoDeviceShared3D (
			) { return (GetStereoDevicePriv3D()); }

		//! Determine if views should be updated to screen after each layer is rendered
		static bool GetUpdateAfterDrawLayer (
			);

		//! Determine if should wait for right mouse-button before zoom/pan
		static bool GetZoomPanWaitForRMB (
			);

		//! Set global options applying to all views.
		static void SetGlobalOptions (
			const VIEWCOMMON& options				//!< Options to set
			);

		//! Set shared stereo parameters for 2D views.
		static void SetStereoDeviceShared2D (
			const STEREODEVICE& StereoDevice
			);

		//! Set shared stereo parameters for 3D views.
		static void SetStereoDeviceShared3D (
			const STEREODEVICE& StereoDevice
			);

		//! Set option for tracking cursor between views
		static void SetTrackCursor (
			bool TrackCursor
			);

		//! Set option to update views after each layer is rendered
		static void SetUpdateAfterDrawLayer (
			bool UpdateAfterDrawLayer
			);

		// Install SML class, called by internal SML function.
		static void SML_ClassInstall (
			SML::CONTEXT *context
			);

		//! Default constructor.
		//! Performs initialization only, must use one of the Create methods to actually
		//! create a usable VIEW.
		VIEW ();

		//! Destructor.
		virtual ~VIEW ();

		//! Add delegate to call when pointer enters view.
		void AddDelegateOnPointerEnter (
			DELEGATE_POINTERENTERLEAVE delegate		//!< Delegate to call
			);

		//! Add delegate to call when pointer leaves view.
		void AddDelegateOnPointerLeave (
			DELEGATE_POINTERENTERLEAVE delegate		//!< Delegate to call
			);

		//! Add delegate to call when pointer moves within view.
		//! Delegate will be passed point in "device canvas" coordinates.
		void AddDelegateOnPointerMove (
			DELEGATE_POINTERMOVE delegate		//!< Delegate to call
			);

		//! Attach GROUPVIEW instance.
		void AttachGroupView (
			GRE::GROUPVIEW *groupview
			);

		//! Attach LAYERVIEW instance.
		void AttachLayerView (
			GRE::LAYERVIEW *layerview
			);

		//! Cancel drawing of this view, usually done within callback.
		//! Will cancel all background jobs for view also.
		void CancelDrawing (
			ERRVALUE err = EUserCancel						//!< Error value to set.
			);

		//! Clear view's message line.
		void ClearMessage ();

		//! Determine if view contains specified layer.
		bool ContainsLayer (
			const GRE::LAYER *layer						//!< Layer to check
			) const;

		//! Convert between "screen units" using specified layer for scale.
		//! @return converted value.
		double ConvertScreenUnits (
			GRE::LAYER *layer,							//!< Layer to use for scale
			double value,									//!< Value to convert
			int iunits,										//!< Input units (SCREENUNIT_...)
			int ounits										//!< Output units (SCREENUNIT_...)
			);

		//! Create view.
		ERRVALUE Create (
			VIEWABLE *viewable,
			int iwidth,
			int iheight,
			CREATEFLAGS createflags = CREATEFLAG_None
			);

		//! Get DataTip selection mode.
		DATATIPSELECTION DataTipGetSelection (
			) const { return (m_DataTipSelection); }

		//! Get DataTip string from specified point.
		//! The images array will be filled in if there are any layers which
		//!	produce image thumbnails for their data tips.  The string will
		//!	have {~IMG} tags controlling where the images get rendered.
		//!	The MGUI::TOOLTIP class takes care of rendering the images
		//! If bForceHTML is true, the resuting text will be converted to 
		//! an HTML fragment (no html, head or body tag) suitable for inserting
		//! into another HTML document.  In this case, images will show up as
		//! img tags with the source set to 'inline:img#' (where # is the
		//! index into the images array)
		ERRVALUE DataTipGetString (
			const DPOINT2D& point,						//!< Point to obtain DataTip for
			MISTRING& string,								//!< String to fill in, will be cleared first
			SPATMOD::IMAGE::STAGE_ARRAY& images,	//!< Images returned for image datatips
			FINDFLAGS findflags,							//!< FINDFLAG_ViewCoords or FINDFLAG_DisplayCoords
			int maxlines = 0,								//!< Approximate maximum number of lines, 0 for no maximum
			DATATIPSELECTION selection = DATATIPSELECTION_FromView,
			bool bForceHTML = false,					//!< Force the output to be an HTML fragment
			MGUI::CTRL_TOOLTIP* tooltip = 0 			//!< Tooltip to pass to Display Control Script, OnViewDataTipShowRequest() (will use temporary if not specified)
			);

		//! Set DataTip popup delay.
		void DataTipSetDelay (
			INT32 delay										//!< DataTip popup delay in milliseconds
			) { v_DataTipSetDelay(delay); }

		//! Set distance cursor must move to hide DataTip.
		void DataTipSetPixelDelta (
			int PixelDelta									//!< Movement distance in screen pixels
			) { v_DataTipSetPixelDelta(PixelDelta); }

		//! Set DataTip selection mode.
		//! Does not affect currently visible DataTip if any.
		void DataTipSetSelection (
			DATATIPSELECTION selection					//!< Selection mode to set
			) { m_DataTipSelection = selection; }

		//! Destroy GROUPVIEW instance for specified group and this view.
		void DestroyGroupView (
			GRE::GROUP *group
			);

		//! Destroy LAYERVIEW instance for specified layer and this view.
		void DestroyLayerView (
			GRE::LAYER *layer
			);

		//! Detach view from viewable.
		//! Intended for use in viewable destructor only, other use may cause crash.
		void Detach (
			) { m_viewable = 0; }

		//! Detach GROUPVIEW instance.
		void DetachGroupView (
			GRE::GROUPVIEW *groupview
			);

		//! Detach LAYERVIEW instance.
		void DetachLayerView (
			GRE::LAYERVIEW *layerview
			);

		//! Disable redraw for this view.
		void DisableRedraw (
			) { ++m_DisableRedraw; }

		//! Draw the view, may result in other views being redrawn also.
		ERRVALUE Draw (
			);

		//! Draw specified group in view.
		ERRVALUE DrawGroup (
			GRE::GROUP *group,								//!< Group, must be contained in view
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Draw view if changed since last redraw.
		//! @return TRUE if did redraw, FALSE if not, < 0 if error.
		int DrawIfNeeded (
			);

		//! Draw view if auto-update is on.
		ERRVALUE DrawIfAutoUpdate (
			) { if (m_RedrawOnAnyChange) return (Draw()); return (0); }

		//! Draw specified layer in view.
		ERRVALUE DrawLayer (
			GRE::LAYER *layer,								//!< Layer to draw
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Draw single element from specified layer.
		ERRVALUE DrawLayerElement (
			GRE::LAYER *layer,								//!< Layer
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,									//!< Element number
			GRE::RENDERTARGET Target,					//!< Target buffer(s) to render to
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Draw set of elements for specified layer.
		ERRVALUE DrawLayerElements (
			GRE::LAYER *layer,							//!< Layer
			ELEMTYPE ElemType,							//!< Element type
			const UNSIGNEDSET *pElemSet,				//!< Set of elements to draw, NULL for all
			GRE::RENDERTARGET Target,					//!< Target buffer(s) to render to
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Draw highlighed elements for specified layer in view.
		ERRVALUE DrawLayerHighlighted (
			GRE::LAYER *layer,								//!< Layer to draw
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Draw the margin lines.
		//! This function does nothing if not in hardcopy mode.
		void DrawMargins (
			MGD::CONTEXT* gc,	 							//!< Context to draw with
			const TRANS2D_AFFINE *pTransViewDevice = 0
			);

		//! Draw specified layer overlay in view
		ERRVALUE DrawLayerOverlay (
			GRE::LAYER *layer,								//!< Layer to draw
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Draw view without refreshing other views.
		ERRVALUE DrawThisOnly (
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Edit background, highlight, active colors for view.
		//! @return TRUE if colors changed, FALSE if not, < 0 if error.
		int EditColors (
			);

		//! Enable redraw of this view.
		void EnableRedraw (
			) { if (m_DisableRedraw > 0) --m_DisableRedraw; }

		//! Find closest element to specified point in view.
		//! @return Element number, -1 if none found, < -1 if error.
		ELEMNUM FindClosestLayerElement (
			GRE::LAYER *layer,								//!< Layer to search
			int component,									//!< Layer component index (not the element type)
			const DPOINT2D& point,						//!< Point to search around
			double *distret,								//!< Distance passed/returned, NULL if don't need
			GRE::FINDFLAGS findflags					//!< Flags for searching
			);

		//! Find closest group to specified point in view.
		//! @return pointer to group or NULL if none found.
		GRE::GROUP* FindPointGroup (
			const DPOINT2D& point,						//!< Point to search
			GRE::FINDFLAGS findflags,					//!< Flags for searching
			const SPATREF::COORDREFSYS *pPointCRS = NULL		//!< CoordRefSys for point if FINDFLAG_MapCoords specified
			) const;

		bool GetAutoRedrawSolid (
			) const { return (m_AutoRedrawSolid); }

		//! Return name of background color for use in INI file.
		const char* GetBackgroundColorIniName (
			) const;

		//! Get canvas clipping rectangle.
		const LRECT2D& GetCanvasClipRect (
			) const { return (m_ClipRect); }

		//! Get portion of canvas extents actually used based on zoom factor.
		void GetCanvasExtentsUsed (
			LRECT2D& extents,								//!< Extents in 'canvas' coordinates returned
			bool UseNext = false							//!< Use 'next' position instead of 'last' position
			) const;

		//! Get clipping region for group in view.
		//! @return false if not in visible area, true if visible.
		bool GetClipRegion (
			GRE::GROUP *group,								//!< Group, NULL for overall clipping region
			REGION2D& ClipRegion							//!< Clipping region in device coordinates returned
			);

		//! Get color for specified usage.
		const COLOR& GetColor (
			COLORUSE coloruse								//!< Color usage
			) const { return (m_Colors[coloruse]); }

	#ifdef USE_MGUI
		//! Get suitable parent for dialog child of view window.
		MDLGPARENT GetDlgParent (
			) const { return (v_GetDlgParent()); }
	#endif

		//! Get height (Y extent) of drawable area.
		//! @return Height in pixels.
		UINT32 GetDrawingAreaHeight (
			) const { return (m_height); }

		//! Get width (X extent) of drawable area.
		//! @return Width in pixels.
		UINT32 GetDrawingAreaWidth (
			) const { return (m_width); }

		//! Get MGD::DEVICE for drawing.
		//! If SetDrawingDevice has been used then the device set will be returned.
		//! Otherwise will be same as GetDrawingDeviceMemory().
		MGD::DEVICE* GetDrawingDevice (
			bool UseStereo = false						//!< Use dual frame buffer stereo device if available and view is set to draw stereo
			) const;

		//! Get MGD::DEVICE for drawing to 'memory'.
		MGD::DEVICE* GetDrawingDeviceMemory (
			bool UseStereo = false						//!< Use dual frame buffer stereo device if available and view is set to draw stereo
			) const;

		//! Get MGD::DEVICE for drawing to 'screen'.
		MGD::DEVICE* GetDrawingDeviceScreen (
			) const { return (v_GetDrawingDeviceScreen()); }

		//! Determine if want drawing in stereo.
		bool GetDrawStereo (
			) const { return (m_DrawStereo); }

		//! Determine if only drawing wireframes for 3D layers.
		bool GetDrawWireframe3D (
			) const { return (m_DrawWireframe); }

		//! Get full extents of viewable object including margins.
		const DRECT2D& GetFullExtents (
			) const { return (m_FullExtents); }

		//! Get geo-lock settings for view.
		const GEOLOCKSETTINGS& GetGeoLock (
			) const { return (m_GeoLock); }

		//! Get group used suitable for georeference.
		//! @return NULL if none suitable.
		GRE::GROUP* GetGeorefGroup (
			) const;

		//! Get viewable or active group.
		//! If viewable is a layout, the active group will be returned.
		GRE::GROUP* GetGroup (
			) const;

		//! Get GROUPVIEW pointer.
		GRE::GROUPVIEW* GetGroupView (
			const GRE::GROUP *group
			) const;

		//! Get last-drawn position.
		const VIEWPOS& GetLastPosn (
			) const { return (m_LastPosn); }
	
		//! Get LAYERVIEW pointer.
		GRE::LAYERVIEW* GetLayerView (
			const GRE::LAYER *layer
			) const;

		//! Get viewable layout.
		GRE::LAYOUT* GetLayout (
			) const;

		//! Get current panning margin.
		//! @return Margin in meters.
		double GetMargin (
			) const { return (m_MarginMeters); }

		//! Get last message shown on status line.
		const MISTRING& GetMessage (
			) const { return (m_LastMessage); }

		//! Get name of this view.
		void GetName (
			MISTRING& name									//!< Name returned
			) const;

		//! Get whether need to call NotifyDrawElementBegin/End before/after drawing each element.
		bool GetNeedNotifyDrawElement () const
			{ return (m_NeedNotifyDrawElement); }

		bool GetNeedsRedraw (
			) const { return (m_NeedsRedraw); }

		//! Get next view of same object as this view.
		//! @return next view or 0 if no more views.
		VIEW* GetNext (
			) const;

		VIEW* GetNextOfAll (
			) const { return (m_nextview); }

		const VIEWPOS& GetNextPosn (
			) const { return (m_NextPosn); }

		//! Get padding value in pixels for scrolling off edge of object extent.
		INT16 GetPadPixels (
			) const { return (m_PadPixels); }

		//! Get pixel size of rendering device in millimeters.
		double GetPixelSizeMM (
			) const;

		//! Get pixel size of rendering device in millimeters.
		void GetPixelSizeMM (
			DPOINT2D& pixelsize
			) const;

		//! Get current projection accuracy.
		GRE_PROJACCUR GetProjAccur (
			) const { return (m_ProjAccur); }

		//! Determine if should redraw views on any change
		bool GetRedrawOnAnyChange (
			) { return (m_RedrawOnAnyChange); }

		//! Get current scale at which view is rendered.
		ERRVALUE GetScale (
			double *xscale,								//!< X scale returned
			double *yscale									//!< Y scale returned
			);

		//! Get current scale at which view is rendered.
		double GetScale (
			) const { return (m_LastPosn.GetMapScale()); }

		//! Get "size" of viewable object in "ground" meters.
		ERRVALUE GetSizeMeters (
			DPOINT2D& size									//!< Size in meters returned
			) const;

		MISTATUSMANAGER* GetStatusManager (
			) const { return (m_pStatusMgr); }

		//! Get stereo device parameters for this view.
		const STEREODEVICE& GetStereoDevice () const;

		//! Get stereo rendering parameters for this view.
		const STEREORENDER& GetStereoRender () const;

		bool GetStopDrawOnError (
			) const { return (m_StopDrawOnError); }

		double GetTimeLastRefresh (
			) const { return (m_TimeLastRefresh); }

		//! Get transformation from display to group.
		void GetTransDispToGroup (
			const GRE::GROUP *group,						//!< Group
			TRANS2D_AFFINE& DispToGroup				//!< Transformation returned
			) const;

		//! Get transformation from layer to display coordinates in TRANS2D_MAPGEN.
		//! @return TRUE if layer within view, FALSE if not, < 0 if error.
		int GetTransLayerDisp (
			const GRE::LAYER *layer,						//!< Layer to obtain transformation for
			TRANS2D_MAPGEN& trans						//!< Transformation returned
			) const;

		//! Get transformation from layer to view coordinates in TRANS2D_MAPGEN.
		ERRVALUE GetTransLayerView (
			const GRE::LAYER *layer,						//!< Layer to obtain transformation for
			TRANS2D_MAPGEN& trans						//!< Transformation returned
			) const;

		//! Get tranformation from view to map coordinates in TRANS2D_MAPGEN
		ERRVALUE GetTransToMap (
			TRANS2D_MAPGEN& trans,						//!< Transformation parameters returned
			bool dooutput,									//!< Use output section of TransParm
			const GRE::GROUP *group = 0					//!< Reference group if desired, NULL for default
			) const;

		//! Get transformation from view to display as TRANS2D_AFFINE.
		//! @return Affine tranformation.
		const TRANS2D_AFFINE& GetTransToDisp (
			) const { return (m_LastPosn.GetTransViewDevice()); }

		//! Get object this view is of.
		VIEWABLE* GetViewable (
			) const { return (m_viewable); }

		//! Get type of object being viewed.
		OBJECT::TYPE GetViewableType (
			) const;

		VIEWFLAGS GetViewFlags (
			) const { return (m_viewflags); }

		//! Get view number for visibility check.
		int GetViewNum (
			) const;

		//! Get 3D viewpoint for this view.
		//! @return viewpoint pointer or 0 if not a 3D view.
		VIEWPOINT3D* GetViewPoint3D (
			) const { return (m_ViewPoint3D); }

		//! Get extents of viewable object which are visible.
		void GetVisibleExtents (
			DRECT2D& extents,								//!< Extents returned
			bool UseNext = false							//!< Use 'next' position instead of 'last' position
			) const;

		//! Determine if in wireframe mode
		bool GetWireframeMode (
			) const;

		//! Get current zoom factor for this view.
		const double& GetZoom (
			) const { return (m_NextPosn.GetZoom().x); }

		//! Get ZVIEW for surface value access and stereo drawing.
		//! @return Pointer to ZVIEW if available, NULL if not.
		const ZVIEW* GetZView () const;

		//! Determine if view contains any 3D groups.
		bool Has3D () const;

		//! Determine if view has any visible objects.
		bool HasObjects () const;

		//! Highlight elements in view using specified point.
		//! @return 'true' if any element highlighted, false if not.
		bool HighlightElementsByPoint (
			const DPOINT2D& pt,							//!< Point to use for finding elements
			bool PointInDisplayCoords,					//!< 'true' if point in "display" coordinates, 'false' if in "view" coordinates.
			GRE::HIGHLIGHTMODE HighlightMode = GRE::HIGHLIGHTMODE_NotSet,	//!< Highlight operation
			bool ShowDftTables = false					//!< Show default table for each layer highlighted element
			);

		//! Invalidate all layers so will force to redraw.
		void InvalidateAllLayers (
			);

		//! Determine if view is of a 3D group.
		bool Is3D (
			) const;

		bool IsAllowedDiffZoom (
			) const { return ((m_viewflags & VIEWFLAG_AllowDiffZoom) != 0); }

		bool IsBeingDrawn (
			) const { return (m_DoingRedraw); }

		bool IsBeingRestored (
			) const { return (m_DoingRestore); }

		//! Determine if view is geolocked.
		bool IsGeoLocked (
			) const { return (m_GeoLocked); }

		//! Determine if view is geolocked.
		bool IsGoogleLocked (
			) const { return (m_GoogleLocked); }

		//! Determine if group is set to be visible in view.
		bool IsGroupVisible (
			const GRE::GROUP *group
			) const;

		//! Determine if is "interactive" view.
		bool IsInteractive (
			) const { return (m_IsInteractive); }

		//! Determine if layer is set to be visible in view.
		bool IsLayerVisible (
			const GRE::LAYER *layer						//!< Layer to check
			) const;

		bool IsLegacy (
			) const { return (m_IsLegacy); }

		//! Determine if view is a "locator" window.
		bool IsLocator (
			) const { return ((m_viewflags & VIEWFLAG_Locator) != 0); }

		//! Determine if redraw is disabled.
		bool IsRedrawDisabled (
			) const { return (m_DisableRedraw != 0); }

		//! Notify view of change to 3D viewpoint.
		//! Will invoke callback on view using ACTION_ChangeViewPoint3d or ACTION_ChangeViewPoint3dDragging.
		void NotifyChangeViewPoint3D (
			bool redraw = true,
			bool IsDragging = false
			);

		//! Send notification before drawing element.
		void NotifyDrawElementBegin (
			GRE::LAYER *layer,
			const RVC::ELEMENT& element
			) { v_OnDrawElementBegin(layer,element); }

		//! Send notification after drawing element.
		void NotifyDrawElementEnd (
			GRE::LAYER *layer,
			const RVC::ELEMENT& element
			) { v_OnDrawElementEnd(layer,element); }

		//! Send "exposure" notification to callback list.
		void NotifyExposure (
			const LRECT2D& rect							//!< Rectangle defining area exposed
			);

		//! Notify view that layer has been rendered.
		//! Usually called by worker thread.
		void NotifyLayerRendered (
			GRE::LAYER *layer
			);

		//! Send notification that pointer has entered view.
		void NotifyPointerEnter (
			);

		//! Send notification that pointer has left view.
		void NotifyPointerLeave (
			);

		//! Send notification that pointer has moved within view.
		void NotifyPointerMove (
			const DPOINT2D& point						//!< Point in "device canvas" coordinates
			);

		//! Create overlay buffer.
		//! Do not use delete on returned pointer, use OverlayDestroy to remove.
		//! The overlay device will be automatically created and resized as needed.
		//! The RENDERBUFFER::SetRendered method must be used to indicate that the overlay is valid.
		RENDERBUFFER* OverlayCreate (
			bool AddToFront = false						//! True to add as first overlay, false to add as last
			);

		//! Destroy overlay buffer.
		void OverlayDestroy (
			RENDERBUFFER *& pRenderBuffer				//! Overlay buffer previously obtained via CreateOverlay()
			);

		//! Pan view so that specified layer element is visible.
		ERRVALUE PanToLayerElement (
			GRE::LAYER *layer,								//!< Layer
			ELEMTYPE ElemType,							//!< Element type
			ELEMNUM ElemNum,									//!< Element number
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Reset "cancelled" status
		void ResetCancel (
			) { m_CancelDraw = 0; }

		//! Remove delegate called when pointer enters view.
		void RemoveDelegateOnPointerEnter (
			DELEGATE_POINTERENTERLEAVE delegate		//!< Delegate to remove
			);

		//! Remove delegate called when pointer leaves view.
		void RemoveDelegateOnPointerLeave (
			DELEGATE_POINTERENTERLEAVE delegate		//!< Delegate to remove
			);

		//! Remove delegate called when pointer moves within view.
		void RemoveDelegateOnPointerMove (
			DELEGATE_POINTERMOVE delegate		//!< Delegate to remove
			);

		//! Restore view from rendering buffers.
		//! This will re-composite from the various buffers into the common memory device buffer.
		//! If no such compositing is needed, use RestoreFromMemory instead as will be faster.
		void RestoreFromBuffers (
			bool RenderIfNeeded = true
			);

		//! Restore from off-screen memory.
		//! This does not re-composite from the various layer buffers, use RestoreFromBuffers if
		//! such compositing is needed.
		void RestoreFromMemory (
			const LRECT2D *rect = 0,					//!< Rectangle to restore, 0 for entire visible extent
			bool UseScreenBuffer = false				//!< Copy from screen buffer, should be used only if certain buffer is current
			);

	#ifdef USE_MGUI
		//! Save "quick snapshot" of current view contents.
		ERRVALUE SaveSnapshotQuick (
			);
	#endif

	#ifdef USE_MGUI
		//! Save "snapshot" of current view contents to raster.
		ERRVALUE SaveSnapshotToRaster (
			);
	#endif

		void SetAutoRedrawSolid (
			bool AutoRedrawSolid
			) { m_AutoRedrawSolid = AutoRedrawSolid; }

		void SetBeingRestored (
			bool BeingRestored
			) { m_DoingRestore = BeingRestored; }

		//! Pan view so that specified point in view coordinates is centered.
		//! Panning will not occur if specified point is already in visible extents.
		//! @return TRUE if center changed, FALSE if already visible, < 0 if error.
		int SetCenter (
			const DPOINT2D& pt,							//!< Point in viewable coordinates to pan to
			bool redraw = true							//!< Redraw view if necessary
			);

		//! Pan view so that specified point in layer coordinates is centered.
		void SetCenterByLayer (
			GRE::LAYER *layer,								//!< Layer
			const DPOINT2D& pt,							//!< Point in layer coordinates
			bool redraw = true							//!< Redraw view
			);

		//! Reset clipping to full extent of drawing area.
		void SetClipNone (
			);

		//! Set clipping rectangle for view in display coordinates.
		void SetClipRect (
			const LRECT2D& cliprect						//!< Clipping rectangle
			);

		//! Set color for specified use.
		void SetColor (
			COLORUSE coloruse,							//!< Color use
			const COLOR& color							//!< Color to set
			);

		void SetDrawingAreaSize (
			INT32 width,
			INT32 height
			);

		//! Set MGD::DEVICE for drawing.
		//! Should not be used if CreateX() was used to create the view.
		void SetDrawingDevice (
			MGD::DEVICE *gdisp
			);

		void SetDrawStereo (
			bool DrawStereo
			);

		void SetDrawWireframe3D (
			bool DrawWireframe
			) { m_DrawWireframe = DrawWireframe; }

		//! Set whether or not view is geolocked to other views in same process
		void SetGeoLocked (
			bool GeoLocked = true,
			bool AutoScale = true
			);

		//! Set geo-locking values.
		void SetGeoLock (
			const GEOLOCKSETTINGS& GeoLockSettings
			) { m_GeoLock = GeoLockSettings; }

		void SetGoogleLocked (
			bool Locked
			) { m_GoogleLocked = Locked; }

		//! Set additional panning margin.
		//! This allows the user to pan past the "edge" of the viewable object when zoomed.
		//! Margins are currently supported for 2D 'group' views only.
		void SetMargin (
			double MarginMeters
			) { m_MarginMeters = MarginMeters; m_NeedsRedraw = true; }

		//! Set whether need to call NotifyDrawElementBegin/End before/after drawing each element.
		//! Default is that notification is not needed unless this method is called.
		void SetNeedNotifyDrawElement (
			bool NeedNotify = true
			) { m_NeedNotifyDrawElement = NeedNotify; }

		void SetNeedsRedraw (
			) { m_NeedsRedraw = true; }

		//! Set next "place" to view and optionally redraw.
		void SetNextPlace (
			const VIEWPLACE& NextPlace,
			bool redraw = true
			);

		//! Set next view position.
		void SetNextPosn (
			const VIEWPOS& NextPosn,
			bool redraw = true
			) { m_NextPosn = NextPosn; m_NeedsRedraw = true; if (redraw) Draw(); }

		void SetPadPixels (
			int pp
			) { m_PadPixels = static_cast<INT16>(bound(pp,0,100)); }

		void SetProjAccur (
			GRE_PROJACCUR ProjAccur
			) { m_ProjAccur = ProjAccur; }

		//! Set option to redraw view on any change.
		void SetRedrawOnAnyChange (
			bool RedrawOnAnyChange,
			bool redraw = true						//! Redraw if changed and setting turned on
			);

		//! Set scale to draw view at.
		ERRVALUE SetScale (
			double NewMapScale,
			bool redraw = true
			);

		//! Set scale to draw specified reference layer at maximum detail.
		ERRVALUE SetScaleMaxDetail (
			GRE::LAYER *ScaleRefLayer,
			bool redraw = true
			);

		//! Set scale to draw default reference layer at maximum detail.
		ERRVALUE SetScaleMaxDetail (
			bool redraw = true
			) { return (SetScaleMaxDetail(NULL,redraw)); }

		void SetStatusManager (
			MISTATUSMANAGER *pStatusMgr
			) { m_pStatusMgr = pStatusMgr; }

		//! Set stereo parameters for this view.
		//! If not set will use shared parameters.
		void SetStereoDevice (
			const STEREODEVICE& StereoDevice
			);

		//! Set stereo rendering parameters for this view.
		void SetStereoRender (
			const STEREORENDER& StereoRender
			);

		//! Set whether to stop drawing if error occurs.
		void SetStopDrawOnError (
			bool StopDrawOnError = true				//!< true to stop, false to display error and continue
			) { m_StopDrawOnError = StopDrawOnError; }
	
		void SetViewFlags (
			VIEWFLAGS viewflags,
			bool redraw = false
			) { m_viewflags = viewflags; if (redraw) Draw(); }

		//! Set view position/scale if geolocked.
		//! @return true if zoom or position changed, false if not.
		bool SetViewFromGeoLock (
			const VIEW *refview=0					//!< Reference view, default to first
			);

		void SetViewNum (
			int ViewNum
			) { m_ViewNum = ViewNum; }

		//! Set to draw wireframes instead of groups
		void SetWireframeMode (
			bool mode,
			bool redraw = true							//!< Redraw now if true
			);

		//! Set zoom factor for view.
		int SetZoom (
			double zoom,									//!< Zoom factor, 1.0 = full extents
			bool redraw = true							//!< Redraw view
			);

		//! Set zoom factor for view.
		int SetZoom (
			const DPOINT2D& zoom,
			bool redraw = true							//!< Redraw view
			);

		//! Set zoom to view full extents.
		int SetZoomFull (
			bool redraw = true							//!< Redraw view
			) { return (SetZoom(.99,redraw)); }

		//! Test if user has pressed "Cancel/Skip" button
		int TestCancel (
			) { return (v_TestCancel()); }

		//! Translate canvas device coordinate to view, optionally accounting for stereo settings.
		DPOINT2D TransDeviceToView (
			INT32 x,											//!< Device X coordinate
			INT32 y,											//!< Device Y coordinate
			bool bApplyStereo								//!< Whether to adjust for stereo mode
			) const;

		//! Translate 'display' coordinates to group 'map' coordinates.
		ERRVALUE TransDispToGroup (
			GRE::GROUP *group,							//!< Group
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			) const;

		//! Translate 'display' coordinates to 'layer' coordinates.
		ERRVALUE TransDispToLayer (
			GRE::LAYER *layer,								//!< Layer to transform to
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			);

		//! Translate 'display' coordinates to 'view' coordinates.
		//! 'View' coordinates are either 'group' or 'layout' coordinates and are
		//! generally considered to be arbitrary.
		void TransDispToView (
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			) const;

		//! Translate group 'map' coordinates to 'display' coordinates.
		ERRVALUE TransGroupToDisp (
			GRE::GROUP *group,								//!< Group
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			) const;

		//! Translate group 'map' coordinates to 'view' coordinates.
		//! 'View' coordinates are either 'group' or 'layout' coordinates and are
		//! generally considered to be arbitrary.
		ERRVALUE TransGroupToView (
			GRE::GROUP *group,								//!< Group
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			) const;

		//! Translate 'layer' coordinates to 'display' coordinates.
		ERRVALUE TransLayerToDisp (
			GRE::LAYER *layer,
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			);

		//! Translate 'layer' coordinates to 'view' coordinates.
		//! 'View' coordinates are either 'group' or 'layout' coordinates and are
		//! generally considered to be arbitrary.
		ERRVALUE TransLayerToView (
			GRE::LAYER *layer,
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			);

		//! Translate 'view' coordinates to 'display' coordinates.
		//! 'View' coordinates are either 'group' or 'layout' coordinates and are
		//! generally considered to be arbitrary.
		void TransViewToDisp (
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			) const;

		//! Translate 'view' coordinates to group 'map' coordinates.
		//! 'View' coordinates are either 'group' or 'layout' coordinates and are
		//! generally considered to be arbitrary.
		ERRVALUE TransViewToGroup (
			GRE::GROUP *group,								//!< Group
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			) const;

		//! Translate 'view' coordinates to 'layer' coordinates.
		//! 'View' coordinates are either 'group' or 'layout' coordinates and are
		//! generally considered to be arbitrary.
		ERRVALUE TransViewToLayer (
			GRE::LAYER *layer,
			const DPOINT2D *ipoints,					//!< Source point array
			DPOINT2D *opoints,							//!< Target point array
			int numpoints									//!< Number of points to convert
			);

	#ifdef USE_MGUI
		//! Get TreeView pointer.
		GRE::CTRL_TREEVIEW* TreeViewGetPtr (
			) const { return (m_pTreeView); }
	#endif

	#ifdef USE_MGUI
		//! Set TreeView pointer.
		//! Used by GRE CTRL_TREEVIEW implementation, should not be called directly.
		void TreeViewSetPtr (
			GRE::CTRL_TREEVIEW *pTreeView
			) { m_pTreeView = pTreeView; }
	#endif

		//! Update viewable extents.
		ERRVALUE UpdateExtents (
			);

		void UpdateLastPosn (
			);

		//! Update text on message line.
		void UpdateMessage (
			const MISTRING& string
			);

		int UpdatePosition (
			);

		//! Update script user interface controls for interactive view.
		void UpdateScriptUI (
			) { v_UpdateScriptUI(); }

		//! Update view's "type name" if viewable object's type name may have changed.
		void UpdateTypeName (
			);

		//! Update user interface controls for interactive view.
		void UpdateUI (
			) { v_UpdateUI(); }

		//! Determine if using separate render devices for each layer/group.
		bool UseSeparateRenderDevices (
			);

		//! Zoom to extent of specified group.
		int ZoomToGroup (
			GRE::GROUP *group,
			bool redraw = true
			);

		//! Zoom to extent of specified rectangle in CRS used to render group.
		int ZoomToGroupRect (
			GRE::GROUP *group,
			const DRECT2D& rect,
			bool redraw = true
			);

		//! Zoom to extent of specified layer.
		int ZoomToLayer (
			GRE::LAYER *layer,
			bool redraw = true
			);

		//! Zoom to extent of specified rectangle in layer coordinates.
		int ZoomToLayerRect (
			GRE::LAYER *layer,
			const DRECT2D& rect,
			bool redraw = true
			);

		//! Zoom to extent of specified rectangle in associated viewable coordinates.
		ERRVALUE ZoomToViewableRect (
			const DRECT2D& rect,						//!< Rectangle to zoom to
			bool redraw = true						//!< True to redraw view
			);

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// SML interface methods.
		static const SML::CLASSMEMBER* SML_GetClassMembers();
		static void SML_ViewDestroy (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);
		#endif

		typedef void DRAWSTATE;

		void ClearNeedsRedraw (
			) { m_NeedsRedraw = false; }

		//! Get tracking cursor position in canvas coordinates.
		const WPOINT2D& CursorTrackGetPos () const
			{ return (m_TrackPos); }

		//! Determine if has tracking cursor position.
		bool CursorTrackHasPos () const
			{ return (m_HaveTrackPos); }

		//! Draw background.
		void DrawBackground ();

		//! Begin drawing - must call DrawEnd in all cases.
		//! Sends notification via callbacks and virtual method.
		//! @return "State" pointer - must be passed to DrawEnd
		DRAWSTATE* DrawBegin (
			GRE::DRAWFLAGS drawflags
			);

		//! End drawing - must be called if DrawBegin used in all cases.
		//! Sends notification via callbacks and virtual method.
		void DrawEnd (
			GRE::DRAWFLAGS drawflags,		
			DRAWSTATE *state						//!< "State" pointer returned from DrawBegin
			);

		ERRVALUE DrawViewable (
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		//! Get view number.
		int GetViewNumRaw (
			) const { return (m_ViewNum); }

		//! Overlay updated layers on display.
		void OverlayUpdatedLayers ();

		//! Set view position directly.
		//! Does not set that needs redraw.
		void SetViewPosns (
			const VIEWPOS& ViewPos
			);

		//! \internal
		void SetInteractive (
			) { m_IsInteractive = true; }

	#ifdef USE_MGUI
		virtual void v_CursorTrackDraw ();
		virtual void v_CursorTrackRestore ();
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef GRE::VIEW THISCLASS;
		typedef std::map<GRE::GROUP*,GRE::GROUPVIEW*> GROUPVIEWMAP;
		typedef std::map<GRE::LAYER*,GRE::LAYERVIEW*> LAYERVIEWMAP;

		VIEWABLE *m_viewable;						// Object being viewed
		VIEW *m_nextview;								// Pointer to "next" view if multiple views are in use
		MGD::DEVICE *m_gdisp;						// Generic display handle for drawing
		VIEWFLAGS m_viewflags;
		COLOR m_Colors[COLORUSE_COUNT];
		VIEWPOS m_LastPosn;							// Position as of last redraw
		VIEWPOS m_NextPosn;							// Position which will be used for next redraw
		DRECT2D m_FullExtents;						// Overall extents of VIEWABLE in "view coordinates"
		MISTATUSMANAGER *m_pStatusMgr;
		VIEWPOINT3D *m_ViewPoint3D;				// 3-D viewpoint if viewing single 3-D group
		int m_ViewNum;									// View number for window title
		int m_CancelDraw;								// Set by TestCancel to reason redraw was cancelled (usually EUserCancel)
		WPOINT2D m_TrackPos;							// Position for cursor when tracking from other views
		GRE_PROJACCUR m_ProjAccur;					// Projection accuracy
		INT32 m_width;									// Width of drawing area in pixels
		INT32 m_height;								// Height of drawing area in pixels
		INT16 m_PadPixels;
		UINT8 m_DisableRedraw;						// If > 0 redraws for exposures are disabled
		char m_TypeNameStr[32];
		GEOLOCKSETTINGS m_GeoLock;
		LRECT2D m_ClipRect;							// Clipping rectangle in display coordinates
	#ifdef USE_MGUI
		GRE::DLG_VIEWZOOM *m_pDlgViewZoom;
		GRE::CTRL_TREEVIEW *m_pTreeView;
		MGUI::TIMER m_TimerOverlayUpdate;
	#endif
		DATATIPSELECTION m_DataTipSelection;
		GROUPVIEWMAP m_GroupViewMap;
		LAYERVIEWMAP m_LayerViewMap;
		MGD::DEVICE_MEM32RGBA *m_pDeviceWork;	// Device for rendering to before copying to renderbuffer
		MISTRING m_LastMessage;						// Last message string
		double m_TimeLastRefresh;
		double m_MarginMeters;						// Maximum margin in meters between view extents and viewable extents.
		STEREODEVICE *m_pStereoDevice;
		mutable STEREORENDER *m_pStereoRender;
		bool m_NeedsRedraw;							// Flag, set if must redraw all on next expose event
		bool m_DoingRedraw;							// Set while redraw is being performed
		bool m_AutoRedrawSolid;						// (3D) Auto-redraw in solid-rendering mode
		bool m_DoingRestore;							// Set while in Restore to prevent looping
		bool m_DrawWireframe;
		bool m_DrawStereo;							// Draw layers in stereo if supported
		bool m_HaveTrackPos;
		bool m_GeoLocked;
		bool m_IsInteractive;						// View is being directly interacted with by user
		bool m_ClipRectUsed;							// Set if clipping rectangle used
		bool m_StopDrawOnError;						// Stop drawing if error occurs
		bool m_CanRestore;							// Set during drawing if has something to restore
		bool m_RedrawOnAnyChange;					// Redraw view on any change
		bool m_IsLegacy;
		bool m_GoogleLocked;
		bool m_NeedNotifyDrawElement;
		MILIST<DELEGATE_POINTERENTERLEAVE> m_DelegateListOnPointerEnter;
		MILIST<DELEGATE_POINTERENTERLEAVE> m_DelegateListOnPointerLeave;
		MILIST<DELEGATE_POINTERMOVE> m_DelegateListOnPointerMove;
		MITHREAD::QUEUE *m_pThreadQueueRemote;
		MITHREAD::ID m_ThreadQueueIdRemote;
		MITHREAD::CRITICALSECTION m_CritSec;

		static VIEWCOMMON& GetGlobalOptionsPriv ();
		static STEREODEVICE& GetStereoDevicePriv2D ();
		static STEREODEVICE& GetStereoDevicePriv3D ();
			
		static void InitStatic ();

		void AddThreadQueueToPoolRemote ();
		bool CheckRenderBuffersValid ();
		bool CheckRenderBuffersValidGroup (GRE::GROUP *group);
		void ComputeAllGroupLayerTrans ();
	#ifdef USE_MGUI
		void CursorTrackDraw ();
		void CursorTrackRestore ();
		void CursorTrackSetPos (const DPOINT2D& pt);
	#endif

		int DrawLayer2D (								//! Draw specified layer
			GRE::LAYER *layer,								//!< Layer to draw
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		int DrawGroup2D (
			GRE::GROUP *group,
			GRE::DRAWFLAGS drawflags
			);

		int DrawGroup3D (
			GRE::GROUP *group,
			GRE::DRAWFLAGS drawflags
			);

		int DrawGroup3DI (
			GRE::GROUP *group,
			GRE::DRAWFLAGS drawflags,
			MGD::DEVICE *pDevice,
			STEREOVIEW StereoView = STEREOVIEW_None,
			LRECT2D *DevRect = 0
			);

		ERRVALUE DrawGroupBegin (
			GRE::GROUP *group,
			GRE::DRAWFLAGS drawflags
			);

		ERRVALUE DrawGroupMain (
			GRE::GROUP *group,
			GRE::DRAWFLAGS drawflags
			);

		void DrawGroupMatte (
			GRE::GROUP *group,
			MGD::DEVICE *pDevice,
			const TRANS2D_AFFINE& TransViewDevice
			);

		void DrawGroupFromBuffers (
			GRE::GROUP *group
			);

		ERRVALUE DrawLayout (
			GRE::LAYOUT *layout,
			GRE::DRAWFLAGS drawflags = GRE::DRAWFLAG_None
			);

		void DrawLayoutMatte (
			MGD::CONTEXT *gc,
			const TRANS2D_AFFINE& TransViewDevice
			);

		void DrawPageBackground (
			MGD::CONTEXT* gc,
			const TRANS2D_AFFINE& TransViewDevice
			);

		const MGD::DEVICE* GetFirstValidDevice (bool NullIfNone) const;

		MITHREAD::QUEUE& GetThreadQueueRemote ();

		//! Highlight layer elements in group given point.
		//! @return 'true' if any element highlighted, false if not.
		bool HighlightElementsByPoint (
			GRE::GROUP *group,							//!< Group to highlight in
			const DPOINT2D& pt,							//!< Point in "view" coordinates.
			GRE::HIGHLIGHTMODE HighlightMode,		//!< Highlight operation
			bool ShowDftTables							//!< Show default table for each layer highlighted element
			);

		void OnThreadQueueDone (
			MITHREAD::ID QueueID
			);

		ERRVALUE SetupWorkDevice ();

		void StartQueuedJobsRemote ();

		void UpdateLayerColors (
			GRE::LAYER *layer
			);

		VIEW (const VIEW&);
		VIEW& operator= (const VIEW&);

		friend class TOOL;

		// GRE::OBJECT overrides.
		virtual bool v_Destroy (bool NotifyParent);
		virtual MICONSTRUCT* v_GetIcon () const;
		virtual const char* v_GetTypeName () const;

		#endif // GENERATING_DOXYGEN_OUTPUT

		// VIEW overridables.

		//! Finish view creation.
		virtual void v_CreateFinish ();

		//! Set DataTip popup delay.
		virtual void v_DataTipSetDelay (
			INT32 delay										//!< DataTip popup delay in milliseconds
			);

		//! Set distance cursor must move to hide DataTip.
		virtual void v_DataTipSetPixelDelta (
			int PixelDelta									//!< Movement distance in screen pixels
			);

		virtual ERRVALUE v_Draw ();

		//! Draw this view only.
		//! Intended for internal GRE use only.
		//! Consult source when override for mandatory sequencing.
		virtual ERRVALUE v_DrawThisOnly (GRE::DRAWFLAGS drawflags);

	#ifdef USE_MGUI
		//! Get suitable parent for dialog child of view window.
		//! Default implementation returns NULL.
		virtual MDLGPARENT v_GetDlgParent (
			) const;
	#endif

		//! Get MGD::DEVICE for drawing to 'memory'
		//! Default implementation returns NULL.
		virtual MGD::DEVICE* v_GetDrawingDeviceMemory (
			bool UseStereo								//!< Use dual frame buffer stereo device if available and view is set to draw stereo
			) const;

		//! Get MGD::DEVICE for drawing to 'screen'
		//! Default implementation returns NULL.
		virtual MGD::DEVICE* v_GetDrawingDeviceScreen (
			) const;

		//! Override to return different view number.
		//! Should return -1 for standard view number.
		virtual int v_GetViewNum () const;

		//! Called before each "element" in a geometric layer is drawn.
		virtual void v_OnDrawElementBegin (GRE::LAYER *layer, const RVC::ELEMENT& element);
		//! Called after each "element" in a geometric layer is drawn.
		virtual void v_OnDrawElementEnd (GRE::LAYER *layer, const RVC::ELEMENT& element);
		//! Called before each group in layout is drawn.
		virtual void v_OnDrawGroupBegin (GRE::GROUP *group);
		//! Called after each group in layout is drawn.
		virtual void v_OnDrawGroupEnd (GRE::GROUP *group);
		//! Called before each layer is drawn.
		virtual void v_OnDrawLayerBegin (GRE::LAYER *layer);
		//! Called after each layer is drawn.
		virtual void v_OnDrawLayerEnd (GRE::LAYER *layer);
		//! Called before "viewable" is drawn.
		virtual void v_OnDrawViewableBegin ();
		//! Called after "viewable" is drawn.
		virtual void v_OnDrawViewableEnd ();
		//! Called when background job started.
		virtual void v_OnJobStarted ();
		//! Called when background jobs all finished.
		virtual void v_OnJobsAllDone ();
		//! Called when layer has been rendered by background thread.
		virtual void v_OnLayerRendered (GRE::LAYER *layer);

		//! Restore from off-screen memory.
		//! Default implementation does nothing.
		virtual void v_RestoreFromMemory (
			const LRECT2D *rect,							//!< Rectangle to restore, NULL for full visible extent
			bool UseScreenBuffer = false				//!< Copy from screen buffer, should be used only if certain buffer is current
			);

		virtual ERRVALUE v_TestCancel ();

		//! Update displayed message.
		virtual void v_UpdateMessage (
			const MISTRING& string
			);

		//! Update script user interface for interactive view.
		//! Default implementation does nothing.
		virtual void v_UpdateScriptUI ();

		//! Update user interface for interactive view.
		//! Default implementation does nothing.
		virtual void v_UpdateUI ();

		friend class VIEW_LEGACY;
		friend class VIEW_LEGACY_DATATIP;
		friend class GRE::VIEW_LEGACY_STATUSINFO;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(VIEW::CREATEFLAGS)
#endif

}	// End namespace GRE

//===================================================================================================================

#endif	// INC_GRE_VIEW_H

