/**
 * \file mgui/customui.h
 * \brief User interface customization definitions
 *
 * \if NODOC
 * $Id: customui.h_v 1.4 2003/10/03 19:58:19 linux32build!build Exp $
 *
 * $Log: customui.h_v $
 * Revision 1.4  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.3  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/05/14 21:21:58  mju
 * Allow to set path to customization settings and to reset to default.
 *
 * Revision 1.1  2003/05/13 21:54:42  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_CUSTOMUI_H
#define	INC_MGUI_CUSTOMUI_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

class FILEPATH;

namespace MGUI {

//-------------------------------------------------------------------------------------------------------------------
//!		Class to contain custom settings for current application.
class CUSTOMUI {
	public:

		//! Determine if specified component is "enabled".
		static bool IsComponentEnabled (
			ICID icid									//!< Interface component ID
			);

	#ifdef X_NATIVE
		//! Register widget for dynamic customization (X only).
		static void RegisterWidget (
			Widget w,
			ICID icid
			);
	#endif

		//! Set whether component enabled.
		static void SetComponentEnabled (
			ICID icid,									//!< Interface component ID
			bool enabled								//!< State to set
			);

		//! Set to use standard default files.
		//! This will re-read the settings if necessary and update controls.
		static void SetFileDefault (
			bool reread
			);

		//! Set to use specified file for settings.
		//! This will read the settings if necessary and update controls.
		//! If file does not exist no settings will be changed.  Settings may
		//! be subsequently written to the specified file using WriteSettings().
		static ERRVALUE SetFilePath (
			const FILEPATH& filepath,
			bool reread
			);

		//! Write settings to current 'user' file if necessary.
		static ERRVALUE WriteSettings (
			);

		//! Destructor.
		~CUSTOMUI ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		friend class CUSTOMUI::PRIV;

		static CUSTOMUI& GetInstance ();

		CUSTOMUI ();

		CUSTOMUI (const CUSTOMUI& rhs);
		CUSTOMUI& operator= (const CUSTOMUI& rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI

#endif	// INC_MGUI_CUSTOMUI_H
