/**
 * \file mgui/dlgcontrastparm.h
 * \brief Dialog for editing contrast parameters
 *
 * \if NODOC
 * $Id: dlgcontrastparm.h_v 1.2 2008/09/02 17:53:50 mju Exp $
 *
 * $Log: dlgcontrastparm.h_v $
 * Revision 1.2  2008/09/02 17:53:50  mju
 * Fix doxygen warning.
 *
 * Revision 1.1  2007/03/30 17:47:24  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGCONTRASTPARM_H
#define	INC_MGUI_DLGCONTRASTPARM_H

#ifndef	INC_RVC_RASTER_H
	#include <rvc/raster.h>
#endif

#ifndef	INC_MI32_MCB_H
	#include <mi32/mcb.h>
#endif

class CONTRASTPARM;


//! Add callback for "contrast enhancement" dialog. 
int MxdEditContrastAddCallback (
	MXDHANDLE dhandle, 						//!< Handle created by MxdEditContrastCreate()
	McbCallbackFunc cbfunc,					//!< Callback function
	void *cbdata, 								//!< Callback "hook" data
	UINT32 reasonmask							//!< Reasons for which cbfunc() should get called
	);

//! Close previously opened "contrast enhancement" dialog.
int MxdEditContrastClose (
	MXDHANDLE dhandle							//!< Handle created by MxdEditContrastCreate()
	);

//! Create "contrast enhancement" dialog.
//!
//! The number of entries for "contrastparm" and ObjItemList is dependent on the
//!	raster "display" mode specified by "rastdispmode". 
int MxdEditContrastCreate (
	MDLGPARENT dlgparent,		 							//!< Parent for dialog
	MXDHANDLE *dhandle, 										//!< Dialog handle returned
	int rastdispmode,											//!< Raster "display" mode (RASTDISPMODE_...)
	const RVC::OBJITEMLIST& ObjItemListRasters,		//!< List of object item references
	const RVC::RASTER::COMPONENT *RasterComponents,	//!< Array of raster components, NULL for all defaults
	CONTRASTPARM **PtrsContrastParm,						//!< Array of pointers to contrast parameter structures
	RVC::OBJECTNAME **PtrsObjNameContrast,				//!< Array of pointers to contrast subobject names
	UINT32 dialogflags										//!< MxdFLAG_OpenOnCreate, MxdFLAG_DestroyOnClose
	);

//! Destroy previously created "contrast enhancement" dialog.
int MxdEditContrastDestroy (
	MXDHANDLE dhandle							//!< Handle created by MxdEditContrastCreate()
	);

//! Return true if "contrast enhancement" dialog is open, false if not.
int MxdEditContrastIsOpen (
	MXDHANDLE dhandle							//!< Handle created by MxdEditContrastCreate()
	);

//! Open previously created "contrast enhancement" dialog.
int MxdEditContrastOpen (
	MXDHANDLE									//!< Handle created by MxdEditContrastCreate()
	);

//! Remove callback for "contrast enhancement" dialog.
int MxdEditContrastRemoveCallback (
	MXDHANDLE dhandle, 						//!< Handle created by MxdEditContrastCreate()
	McbCallbackFunc cbfunc,					//!< Callback function
	void *cbdata, 								//!< Callback "hook" data, must be same as when added
	UINT32 reasonmask							//!< Reason mask, must be same as when added
	);

//! Update dialog for external changes to contrast tables.
int MxdEditContrastUpdate (
	MXDHANDLE dhandle, 
	int recompute
	);


#endif	// INC_MGUI_DLGCONTRASTPARM_H
