/**
 * \file mgui/formcolorspread.h
 * \brief MGUI::FORM_COLORSPREAD defns.
 *
 * \if NODOC
 * $Id: formcolorspread.h_v 1.4 2011/11/09 14:59:40 mju Exp $
 *
 * $Log: formcolorspread.h_v $
 * Revision 1.4  2011/11/09 14:59:40  mju
 * *** empty log message ***
 *
 * Revision 1.3  2010/09/02 17:37:28  mju
 * Use colorspread class.
 *
 * Revision 1.2  2010/07/23 14:34:28  mju
 * Add style_Stacked.
 *
 * Revision 1.1  2010/06/11 20:43:12  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMCOLORSPREAD_H
#define	INC_MGUI_FORMCOLORSPREAD_H

#ifndef INC_MGUI_FORM_H
#include <mgui/form.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class COLORSPREAD;
#endif

namespace MGUI {

//! Form to select or define color spread.
class FORM_COLORSPREAD : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum STYLE {
			STYLE_Default =		0x00,
			STYLE_Stacked =		0x01,		//!< Create in "stacked" arrangement
			};

		//! Constructor.
		FORM_COLORSPREAD ();

		//! Destructor.
		virtual ~FORM_COLORSPREAD ();

		//! Create form.
		void Create (
			LAYOUT_PANE_BASE& PaneParent,
			const MISTRING& LabelStr,
			int width = 256,							//!< Form width in pixels
			STYLE style = STYLE_Default,
			LAYOUT_SIZEALIGN SizeAlign = LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get spread values.
		const COLORSPREAD& GetValues () const;

		//! Set delegate to call when any change made.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set spread values.
		void SetValues (
			const COLORSPREAD& values
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		#endif
	};

}	// End namespace MGUI

#endif	// INC_MGUI_FORMCOLORSPREAD_H
