/**
 * \file mgui/toolnode.h
 * \brief MGUI::TOOL_NODE definitions
 *
 * \if NODOC
 * $Id: toolnode.h_v 1.6 2011/12/09 20:23:17 mju Exp $
 *
 * $Log: toolnode.h_v $
 * Revision 1.6  2011/12/09 20:23:17  mju
 * No context in v_draw.
 *
 * Revision 1.5  2011/11/08 14:30:54  mju
 * Remove redundant hdr.
 *
 * Revision 1.3  2008/07/21 16:22:40  scowan
 * Added control form.
 *
 * Revision 1.2  2008/03/07 22:35:01  scowan
 * Added line state of no draw.
 *
 * Revision 1.1  2008/03/07 21:20:25  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOOLNODE_H
#define	INC_MGUI_TOOLNODE_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

#ifndef	INC_MI32_POLYLINE_H
#include <mi32/polyline.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Node adjustment tool with attached lines.
class TOOL_NODE : public MGUI::TOOL_BASE {
	public:

		//! Constructor.
		explicit TOOL_NODE (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_NODE ();

		//! Get current tool node coordinates.
		const DPOINT3D& GetNode (
			POLYLINELIST& LineList
			) const;

		//! Set a percentage of the lines to move in relation to the node being moved.
		//! Adding the vertex to end to line movement will occur when the user moves the node.			
		void SetLinePercentage (
			double LinePercentage
			);

		//! Set marker type and size.
		//! The default marker is MARKER_Plus with a size of 9 pixels.
		void SetMarker (
			MARKER marker,								//!< New marker type
			UINT16 size = 0							//!< New marker size, 0 to retain current size
			);

		//! Set tool node coordinates.
		void SetNode (
			const DPOINT3D& node,
			const POLYLINELIST& LineList
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Enumeration constants
		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside
			};
		struct LINE;
			
		//! Tool settings
		MARKER m_marker;
		UINT16 m_markersize;
		//! Tool position values
		DPOINT3D m_Node;						//!< Tool position
		POLYLINELIST m_LineList;			//!< Lines that are attached to this node
		SIMPLE_ARRAY<LINE> m_LineState;
		//! Values used during interaction
		ZONE m_CursorZone;						//!< Current cursor zone
		double m_LinePercentage;
		
		//! Manual NODE Entry form forward declaration
		class MANUAL_NODE;
		friend class MANUAL_NODE;
		class FORM_NODE;
		FORM_NODE* m_pFormNode;

		//! Methods
		void ComputeCursorZone (const DPOINT2D&);
		void DrawLineMarkers (MGD::CONTEXT*, const POLYLINE&, LINE&);
		void MoveNode (const DPOINT3D& node);
		void UpdateLines (const DPOINT3D& node);

		//! TOOL_BASE overrides.
		virtual MGUI::FORM* v_CreateControlForm (LAYOUT_PANE_BASE& ParentPane, CTRLOPT ctrlopt);
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual bool v_OnPointerButtonDown (MGUI::POINT Point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT Point, POINTERBUTTON button, KEYSTATE keystate);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLNODE_H
