/**
 * \file autorefcount.h <mi32/autorefcount.h>
 * \brief Definitions for AUTOREFCOUNT template.
 *
 * \if NODOC
 * $Id: autorefcount.h_v 1.2 2006/07/27 19:28:46 mju Exp $
 *
 * $Log: autorefcount.h_v $
 * Revision 1.2  2006/07/27 19:28:46  mju
 * Change op-> to return pointer to reference.
 *
 * Revision 1.1  2006/07/27 14:45:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_AUTOREFCOUNT_H
#define	INC_MI32_AUTOREFCOUNT_H

//! Template to manage reference-counted class.
//! Templated class must have AddRef() and Release() methods.
//! If GetExclusive is used then must support copy constructor and have GetRefCount() method.
template <typename _TT> class AUTOREFCOUNT {
	public:

		//! Default constructor
		AUTOREFCOUNT (
			): m_pInstance(new _TT) { if (m_pInstance->GetRefCount() == 0) m_pInstance->AddRef(); }

		//! Copy constructor
		AUTOREFCOUNT (
			const AUTOREFCOUNT<_TT>& rhs
			): m_pInstance(rhs.m_pInstance) { m_pInstance->AddRef(); }

		//! Destructor.
		~AUTOREFCOUNT (
			) { m_pInstance->Release(); }

		//! Assignment.
		AUTOREFCOUNT& operator= (
			const AUTOREFCOUNT<_TT>& rhs
			) {
			if (this != &rhs) {
				m_pInstance->Release();
				m_pInstance = rhs.m_pInstance;
				m_pInstance->AddRef();
				}
			return (*this);
			}

		//! Return constant reference to templated instance.
		const _TT& operator* (
			) const { return (*m_pInstance); }

		//! Return reference to templated instance.
		_TT& operator* (
			) { return (*m_pInstance); }

		//! Return constant pointer to templated instance.
		const _TT* operator-> (
			) const { return (&*m_pInstance); }

		//! Return pointer to templated instance.
		_TT* operator-> (
			) { return (&*m_pInstance); }

		//! Get exclusive instance if multiple references exist.
		//! Will call use copy constructor if current reference count is > 1.
		void GetExclusive (
			) {
			if (m_pInstance->GetRefCount() > 1) {
				m_pInstance = new _TT(*m_pInstance);
				if (m_pInstance->GetRefCount() == 0) m_pInstance->AddRef();
				}
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		_TT *m_pInstance;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_AUTOREFCOUNT_H
