/**
 * \file mi32/imagedimensions.h
 * \brief IMAGEDIMENSIONS class definition
 *
 * \if NODOC
 * $Id: imagedimensions.h_v 1.3 2008/05/05 13:36:59 mju Exp $
 *
 * $Log: imagedimensions.h_v $
 * Revision 1.3  2008/05/05 13:36:59  mju
 * Remove seqentialOptimial stuff.
 *
 * Revision 1.2  2008/05/01 20:25:47  mju
 * Add member for whether sequential processing is optimal.
 *
 * Revision 1.1  2008/05/01 19:40:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_IMAGEDIMENSIONS_H
#define	INC_MI32_IMAGEDIMENSIONS_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif


//! Container for image dimensions.
//! Can be used in SIMPLE_ARRAY as contains no pointers or composite members.
class IMAGEDIMENSIONS {
	public:

		//! Default constructor.
		IMAGEDIMENSIONS (
			):
			m_Sampling(1), m_TotalColumns(0), m_TotalRows(0), m_NumColumnsPerTile(0), m_NumRowsPerTile(0)
			{ }

		//! Get number of columns per tile.
		UINT32 GetNumColumnsPerTile (
			) const { return (m_NumColumnsPerTile); }

		//! Get number of rows per tile.
		UINT32 GetNumRowsPerTile (
			) const { return (m_NumRowsPerTile); }

		//! Get sampling if pyramid tier.
		INT32 GetSampling (
			) const { return (m_Sampling); }

		//! Get total number image columns.
		UINT32 GetTotalColumns (
			) const { return (m_TotalColumns); }

		//! Get total number image rows.
		UINT32 GetTotalRows (
			) const { return (m_TotalRows); }

		//! Set sampling for pyramid tier.
		void SetSampling (
			INT32 Sampling
			) { m_Sampling = Sampling; }

		//! Set tile dimensions.
		void SetTileDimensions (
			UINT32 NumColumnsPerTile,
			UINT32 NumRowsPerTile
			) { m_NumColumnsPerTile = NumColumnsPerTile; m_NumRowsPerTile = NumRowsPerTile; }

		//! Set total and tile dimensions.
		void SetTotalDimensions (
			UINT32 TotalColumns,
			UINT32 TotalRows
			) { m_TotalColumns = m_NumColumnsPerTile = TotalColumns; m_TotalRows = m_NumRowsPerTile = TotalRows; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_Sampling;
		UINT32 m_TotalColumns;
		UINT32 m_TotalRows;
		UINT32 m_NumColumnsPerTile;
		UINT32 m_NumRowsPerTile;
		#endif
	};

#endif	// INC_MI32_IMAGEDIMENSIONS_H
