/**
 * \file md5hash.h <mi32/md5hash.h>
 * \brief Definition for MD5 message-digest algorithm
 *
 * \if NODOC
 * $Id: md5hash.h_v 1.4 2006/04/28 20:18:08 scowan Exp $
 *
 * $Log: md5hash.h_v $
 * Revision 1.4  2006/04/28 20:18:08  scowan
 * Added another finish method.
 *
 * Revision 1.3  2006/04/28 20:02:12  scowan
 * Updated code to be more correct.
 *
 * Revision 1.2  2006/04/27 22:23:35  scowan
 * Added simplification functions.
 *
 * Revision 1.1  2006/02/16 19:02:06  scowan
 * Initial revision
 *
 * \endif
**/

/* MD5C.C - RSA Data Security, Inc., MD5 message-digest algorithm
 */

/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
rights reserved.

License to copy and use this software is granted provided that it
is identified as the "RSA Data Security, Inc. MD5 Message-Digest
Algorithm" in all material mentioning or referencing this software
or this function.

License is also granted to make and use derivative works provided
that such works are identified as "derived from the RSA Data
Security, Inc. MD5 Message-Digest Algorithm" in all material
mentioning or referencing the derived work.

RSA Data Security, Inc. makes no representations concerning either
the merchantability of this software or the suitability of this
software for any particular purpose. It is provided "as is"
without express or implied warranty of any kind.

These notices must be retained in any copies of any part of this
documentation and/or software.
 */

#ifndef	INC_MI32_MD5HASH_H
#define	INC_MI32_MD5HASH_H

#ifndef GENERATING_DOXYGEN_OUTPUT
struct MD5_CTX;
class STRUTF8;
#endif // GENERATING_DOXYGEN_OUTPUT

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

class CLASSLIBEXPORT MD5HASH {
	public:

		//! Default constructor.
		MD5HASH (
			);
			
		//! Destructor.
		~MD5HASH (
			);

		//! Append 'buffer' to the string to be hashed.
		//! Call Finish() to get the hash value
		void Append (
			const STRUTF8& buffer
			);
			
		//! Append 'buffer' to the string to be hashed.
		//! Converts to UTF8 first, then appends to the hash
		//! Call Finish() to get the hash value
		void Append (
			const MISTRING& string
			);
			
		//! Append 'buffer' to the string to be hashed.
		//! Call Finish() to get the hash value
		void Append (
			const UINT8* buffer, 
			INT32 BufferSize
			);
			
		//! Clear the hash.
		void Clear (
			);
			
		//! Finish the hash.
		//! Generate and return the hash value from the appended data
		//! This will call Clear() to reset the hash code
		void Finish (
			unsigned char digest[16]
			);
			
		//! Finish the hash.
		//! Generate and return the hex representaion of the hash value
		//! This will call Clear() to reset the hash code
		//! @return Hex representation of digest buffer
		MISTRING Finish (
			);
			
		//! Generate MD5 hash for STRUTF8 string passed in
		//! @return Hex representation of digest buffer
		static MISTRING GenerateHash (
			const STRUTF8& buffer 
			);

		//! Generate MD5 hash for STRUTF8 string passed in
		//! Converts 'string' to UTF8 before generating the hash
		//! @return Hex representation of digest buffer
		static MISTRING GenerateHash (
			const MISTRING& string
			);

	private:
	   #ifndef GENERATING_DOXYGEN_OUTPUT
		MD5_CTX* m_context;

		MD5HASH (const MD5HASH& rhs);
		MD5HASH& operator= (const MD5HASH& rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#undef	CLASSLIBEXPORT
#endif	//!< INC_MI32_MD5HASH_H
