/**
 * \file mi32/mg2mem1s.h
 * \brief MGD::DEVICE_MEM1_SPARSE definitions.
 *
 * \if NODOC
 * $Id: mg2mem1s.h_v 1.12 2011/11/11 14:30:35 mju Exp $
 *
 * $Log: mg2mem1s.h_v $
 * Revision 1.12  2011/11/11 14:30:35  mju
 * Make context nested.
 *
 * Revision 1.1  2002/11/15 15:47:19  dwilliss
 * Initial revision
 * \endif
**/
 
#ifndef INC_MI32_MG2MEM1S_H
#define INC_MI32_MG2MEM1S_H

#ifndef INC_MI32_MD2BINARY_H
#include <mi32/mg2binary.h>
#endif

#ifndef INC_MI32_BITSETD_H
#include <mi32/bitset.h>
#endif


namespace MGD {

//! Memory-based implementation of MGD::DEVICE for sparse 1-bit visuals.
//!
//! This version is implemented in such a way as to be more memory efficient
//! as it will not allocate memory for lines which are solid.  Not as fast
//! as MGD::DEVICE_MEM1.
class DEVICE_MEM1_SPARSE : public MGD::DEVICE_BINARY {
	public:

		class CONTEXT : public MGD::DEVICE_BINARY::CONTEXT {
			public:
				//! Destructor.
				virtual ~CONTEXT ();

			protected:
				//! Default Constructor 
				CONTEXT (
					MGD::DEVICE_MEM1_SPARSE *pDevice
					);

			private:
		   	#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef DEVICE_BINARY::CONTEXT BASECLASS;

				CONTEXT (const CONTEXT&);
				CONTEXT& operator= (const CONTEXT&);

				friend class MGD::DEVICE_MEM1_SPARSE;

				virtual ERRVALUE v_PutPixel (INT32 x, INT32 y, MGD::PIXEL pixel);
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Constructor.
		DEVICE_MEM1_SPARSE ();

		//! Destructor.
		virtual ~DEVICE_MEM1_SPARSE();

		ERRVALUE Create (
			INT32 height,
			INT32 width,
			bool StartValue = false
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		DEVICE_MEM1_SPARSE (const DEVICE_MEM1_SPARSE&);
		DEVICE_MEM1_SPARSE& operator= (const DEVICE_MEM1_SPARSE&);

		BITSET m_RowConstVal;
		SIMPLE_ARRAY<BITSET*> m_RowBits;
		UINT32 m_width;		// Number of columns

		// DEVICE overrides.
		virtual MGD::CONTEXT* v_CreateContext ();
		virtual ERRVALUE v_FillSpan (MGD::CONTEXT *gc, INT32 x1, INT32 x2, INT32 y);
		// DEVICE_BINARY overrides.
		bool v_GetRange (INT32 x, INT32 y, INT32 x2, INT32& endx) const;
		bool v_GetValue (INT32 x, INT32 y) const;
		void v_SetAll (bool value = true);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End of namespace

#endif	// INC_MI32_MG2MEM1S_H
