/**
 * \file mi32/spovln.h
 * \brief SURFACE_LINE_PROP_CALC definition.
 *
 * \if NODOC
 * $Id: spovln.h_v 1.4 2011/11/07 14:10:26 mju Exp $
 *
 * $Log: spovln.h_v $
 * Revision 1.4  2011/11/07 14:10:26  mju
 * Remove redundant hdrs.
 *
 * Revision 1.3  2010/11/05 14:15:18  mju
 * Remove unused hdr, don't inline ctor/dtor.
 *
 * Revision 1.1  2006/05/26 22:15:28  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_SPOVLN_H
#define	INC_MI32_SPOVLN_H

#ifndef	INC_MI32_LINERAST_H
#include <mi32/linerast.h>
#endif

#ifndef	INC_MI32_FILLPOLY_H
#include <mi32/fillpoly.h>
#endif

#ifndef	INC_MI32_STATUNIVAR_H
#include <mi32/statunivar.h>
#endif

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class POLYLINE;
#endif


//! Calculator of surface properties over polyline.
class SURFACE_LINE_PROP_CALC {
	public:

		struct PROPERTIES {
			double m_Length;
			double m_MinZ;
			double m_MaxZ;
			double m_MinSlope;
			double m_MaxSlope;
			bool m_Full;

			PROPERTIES (
				) : 
				m_Length(0.0),
				m_MinZ(DBL_MAX),
				m_MaxZ(-DBL_MAX),
				m_MinSlope(DBL_MAX),
				m_MaxSlope(-DBL_MAX),
				m_Full(true)
				{ }

			void Clear (
				) {
				m_Length = 0.0;
				m_MinZ = DBL_MAX;
				m_MaxZ = -DBL_MAX;
				m_MinSlope = DBL_MAX;
				m_MaxSlope = -DBL_MAX;
				m_Full = true;
				}

			};

		SURFACE_LINE_PROP_CALC ();

		~SURFACE_LINE_PROP_CALC ();

		void Clear ();

		bool Initialize (
			RVC::OBJITEM& objitem
			);

		bool IsInitialized () { return m_Initialized; };

		ERRVALUE Compute (
			const POLYLINE& polyline,
			PROPERTIES& properties
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::RASTER m_Surface;
		RVC::NULLMASK m_NullMask;
		bool m_Initialized;
		RVC::RASTER::IMAGESCALE m_Scale;
		DPOINT2D m_NRScale;
		DRECT2D m_Extents;
		PROPERTIES* m_Properties;

		void ControlProcess (const double xa, const double ya, const double xb, const double yb, const int mode);
		void SplitProcess (const double xa, const double ya, const double xb, const double yb, const int mode);
		void Process (const double xa, const double ya, const double xb, const double yb);
		bool CalculatePlane (const INT32 column, const INT32 line, const int status, DPOINT3D& plane);
		bool GetElevation(const INT32 column, const INT32 line, double& z);

		#endif		
	};

#endif	// INC_MI32_SPOVLN_H
