/**
 * \file rvc/icmprofile.h 
 * \brief interface for the RVC::ICMPROFILE class.
 *
 * \if NODOC
 * $Id: icmprofile.h_v 1.10 2006/06/28 15:55:50 scowan Exp $
 *
 * $Log: icmprofile.h_v $
 * Revision 1.10  2006/06/28 15:55:50  scowan
 * Changed make to take a non-const objitem.
 *
 * Revision 1.9  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.8  2005/12/09 17:29:35  scowan
 * Updated to use credentials.
 *
 * Revision 1.7  2005/07/11 20:12:31  scowan
 * Fixed make issues.
 *
 * Revision 1.6  2005/01/19 22:20:00  scowan
 * Added support for descriptor.
 *
 * Revision 1.5  2003/10/07 17:32:28  scowan
 * Object virtual method changes.
 *
 * Revision 1.4  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.3  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.2  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/05/13 17:59:30  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::ICMPROFILE rvc/icmprofile.h
//!	\brief Manages RVC::OBJTYPE_ICMProfile RVC objects.
//!	
//!	This class inherits from RVC::BINARY
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_ICMProfile
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_ICMProfile
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- RVC::OBJTYPE_Raster

#ifndef INC_RVC_ICMPROFILE_H
#define INC_RVC_ICMPROFILE_H

#ifndef INC_RVC_BINARY_H
	#include <rvc/binary.h>
#endif

namespace RVC {

class ICMPROFILE : public RVC::BINARY {
	public:

		//! Default constructor
		ICMPROFILE (
			);

		//! Copy constructor
		ICMPROFILE (
			const RVC::ICMPROFILE& rhs
			);

		//! Destructor
		virtual ~ICMPROFILE (
			);

		//! Assignment
		RVC::ICMPROFILE& operator= (
			const RVC::ICMPROFILE& rhs
			);

		//! Make a child rvc binary object from a given parent with a name and description
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a child rvc binary object from a given RVCOBJITEM
		ERRVALUE Make (
			RVC::OBJITEM& ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const char* Source = 0,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
	
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;		//!< Get the default object type
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_ICMPROFILE_H
