/**
 * \file rvc/regioncl.h 
 * \brief interface for the RVC::REGION class.
 *
 * \if NODOC
 * $Id: regioncl.h_v 1.17 2012/01/06 21:03:23 mju Exp $
 *
 * $Log: regioncl.h_v $
 * Revision 1.17  2012/01/06 21:03:23  mju
 * Add setCoordRefSys.
 *
 * Revision 1.16  2007/09/06 17:18:59  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.7  2003/05/30 22:06:31  scowan
 * Added a get extentsd method.
 *
 * Revision 1.1  2000/09/29 20:15:44  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_REGIONCL_H
#define INC_RVC_REGIONCL_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif


namespace RVC {

//! Manages "region" objects.  
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Region.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- Any Geospatial object
//!
//!	Read(), Make() and Write() methods inherited from RVCSIMPLE class.  View RVCSIMPLE docs for inherited methods
class REGION : public RVC::SIMPLE {
	public:

		REGION ();

		REGION (
			const RVC::REGION& rhs
			);

		virtual ~REGION ();

		RVC::REGION& operator= (
			const RVC::REGION& rhs
			);

		//! Get region extents.
		//! @return Region's extents
		const DRECT2D& GetExtents () const;

		//! Get region COORDREFSYS
		//! @return Region's COORDREFSYS
		const SPATREF::COORDREFSYS& GetCoordRefSys () const;

		//! Obtain region, must be read first
		//! @return Region reference
		const REGION2D& GetRegion () const;

		//! Set coordinate reference system.
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& crs			//!< New Coordinate Reference System
			) { m_Region.SetCoordRefSys(crs); }

		//! Set the current region to a new region
		ERRVALUE SetRegion (
			const REGION2D& region
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		REGION2D m_Region;

		// Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		
		// Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif		//!< INC_RVC_REGIONCL_H
