/**
 * \file rvc/uidlgdbrecord.h
 * \brief RVC::UI::DLG_DB_RECORD definitions
 *
 * \if NODOC
 * $Id: uidlgdbrecord.h_v 1.3 2009/09/23 15:35:51 mju Exp $
 *
 * $Log: uidlgdbrecord.h_v $
 * Revision 1.3  2009/09/23 15:35:51  mju
 * Add image field support.
 *
 * Revision 1.2  2009/08/11 17:20:02  mju
 * Add onSizeCanvasStyleSample.
 *
 * Revision 1.1  2008/09/22 19:00:34  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIDLGDBRECORD_H
#define	INC_RVC_UIDLGDBRECORD_H

#ifndef	INC_RVC_UIDLGDBBASE_H
#include <rvc/uidlgdbbase.h>
#endif

#ifndef	INC_RVC_UIFORMDBRECORD_H
#include <rvc/uiformdbrecord.h>
#endif

#ifndef	INC_MGUI_MENUEDIT_H
#include <mgui/menuedit.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef	INC_MGUI_CANVAS_H
#include <mgui/canvas.h>
#endif

#ifndef	INC_MGUI_SASH_H
#include <mgui/sash.h>
#endif

#ifndef	INC_MGUI_STATUSBAR_H
#include <mgui/statusbar.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class DBEDIT_TINSTANCE;
class DBEDIT_FACTORY_DLG;
namespace MGUI {
	class FORM_SPATMOD_IMAGE_VIEW;
	}
#endif

namespace RVC {
namespace UI {
//=====================================================================================================================

class DLG_DB_RECORD : public RVC::UI::DLG_DB_BASE {
	public:

		//! Dialog style.
		enum DLGSTYLE {
			DLGSTYLE_Grid =			RVC::UI::FORM_DB_RECORD::FORMSTYLE_Grid,
			DLGSTYLE_UserDefined =	RVC::UI::FORM_DB_RECORD::FORMSTYLE_UserDefined
			};

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context						//!< SML context to install to
			);

		//! Constructor.
		DLG_DB_RECORD (
			DBEDIT_TINSTANCE *idata,
			DLGSTYLE DlgStyle							//!< Dialog style
			);

		//! Destructor.
		virtual ~DLG_DB_RECORD ();

		//! Get current record reference type and number
		void GetRecordRef (
			DBTABLE::RECORDREFTYPE& RefType,		//!< Reference type returned
			INT32& RefNum								//!< Position number returned
			) const { m_FormRecord.GetRecordRef(RefType,RefNum); }

		//! Get current record reference number.
		INT32 GetRecordRefNum (
			) const { return (m_FormRecord.GetRecordRefNum()); }

		//! Get record reference type.
		DBTABLE::RECORDREFTYPE GetRecordRefType (
			) const { return (m_FormRecord.GetRecordRefType()); }

		//! Get current values.
		ERRVALUE GetValues (
			RVC::DBTABLE::RECORD& record			//!< Values returned
			) { return (m_FormRecord.GetValues(record)); }

		//! Save record with optional prompt to user.
		//! @return EUserCancel if user chooses cancel, or error < 0, TRUE if saved, FALSE if not saved or no change.
		INT32 SaveRecord (
			bool AskUser = true						//!< Whether to prompt user to save or not
			);

		//! Set current record, will read values from table if existing.
		ERRVALUE SetRecordRef (
			DBTABLE::RECORDREFTYPE RefType,		//!< Reference type
			INT32 RefNum								//!< Reference number, -1 for RECORDREFTYPE_New record
			) { return (SetRecordRefLow(RefType,RefNum,true)); }

		//! Set values from record.
		ERRVALUE SetValues (
			const RVC::DBTABLE::RECORD& record,	//!< Values to set
			bool IsChanged								//!< Whether to treat as "changed" for saving
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_DB_RECORD THISCLASS;
		class SETTINGS;

		DBEDIT_TINSTANCE *m_idata;
		const DLGSTYLE m_dlgstyle;
		SETTINGS *m_pSettings;
		RVC::UI::FORM_DB_RECORD m_FormRecord;
		MGUI::CTRL_GROUPBOX m_GbxStyleSampleTop;
		MGUI::CTRL_GROUPBOX m_GbxStyleSampleBottom;
		MGUI::CANVAS m_CanvasStyleSampleTop;
		MGUI::CANVAS m_CanvasStyleSampleBottom;
		MGUI::CTRL_SASH m_SashImageView;
		MGUI::FORM_SPATMOD_IMAGE_VIEW *m_pFormImageView;
		MGUI::MENU m_MenuStyleSample;
		MGUI::MENU m_MenuTable;
		MGUI::MENU_EDIT m_MenuEdit;
		MGUI::MENU m_MenuRecord;
		MGUI::CTRL_TOOLBAR m_ToolBar;
		MGUI::CTRL_STATUSBAR m_StatusBar;
		bool m_AskBeforeSave;
		bool m_IsRelatedOnly;
		bool m_CanToggleAttached;
		bool m_AllowAttachMulti;
		bool m_DidWarnToggleAttach;
		INT32 m_Index;
		INT32 m_FieldNumImage;

		void ComputeIndex ();
		void LoadSettings ();
		void OnChangedFieldValue (INT32 FieldNum);
		void OnCmdEdit (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdRecord (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdRecordToggleAttached (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdStyleSample (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdTable (MGUI::ID cmdid, bool ToggleOn);
		void OnSizeCanvasStyleSample (int, int);
		ERRVALUE SetRecordRefLow (const DBTABLE::RECORDREFTYPE RefType, const INT32 RefNum, const bool CalcIndex);
		void UpdateImage ();
		void UpdateStatus ();
		void UpdateStyleSample ();
		void UpdateTitle ();

		// Overrides from SHELL/DLGSHELL.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnApply ();
		virtual void v_OnClose ();
		virtual void v_OnCloseRequest ();
		virtual void v_OnOpen ();

		// Overrides from DLG_DB_BASE.
		virtual void v_OnChangeActiveList ();
		virtual void v_OnChangeACL ();
		virtual void v_OnChangeDescriptor ();
		virtual ERRVALUE v_OnChangeStyle ();
		virtual ERRVALUE v_Read ();
		virtual ERRVALUE v_SaveSettings ();
		virtual ERRVALUE v_Update (const DBEDIT::EVENT& event);
		virtual ERRVALUE v_Write ();

		// Unimplemented
		DLG_DB_RECORD (const DLG_DB_RECORD&);
		DLG_DB_RECORD& operator= (const DLG_DB_RECORD&);
		#endif
	};

//=====================================================================================================================
}	// End namespace UI
}	// End namespace RVC


#endif	// INC_RVC_UIDLGDBRECORD_H
