/**
 * \file imagefilterselect.h <spatmod/imagefilterselect.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_SELECT definition.
 *
 * \if NODOC
 *
 * $Id: imagefilterselect.h_v 1.2 2006/01/27 22:12:40 mju Exp $
 *
 * $Log: imagefilterselect.h_v $
 * Revision 1.2  2006/01/27 22:12:40  mju
 * Remove unused member.
 *
 * Revision 1.1  2006/01/26 17:31:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERSELECT_H
#define	INC_SPATMOD_IMAGEFILTERSELECT_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to select specified component samples from source image.
//! Note that this can be used to replicate components also.
class FILTER_SELECT : public FILTER {
	public:

		//! Construct with specified sample array and optional pixel type.
		//! The array of source sample indices may contain duplicates.
		FILTER_SELECT (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const UINT16 *SrcSamples,				//!< Sample (indexes) to select from source
			UINT16 NumSamples,						//!< Number of samples to select
			PIXEL::TYPE TgtPixelType = PIXEL::TYPE_Invalid	//!< Target pixel type, if TYPE_Invalid will determine from selected samples
			);

		// Destructor.
		virtual ~FILTER_SELECT ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		UINT16 m_NumSamples;
		UINT16 *m_SrcSamples;
		PIXEL::TYPE m_TgtPixelType;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_SELECT (const FILTER_SELECT&);
		FILTER_SELECT& operator= (const FILTER_SELECT&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERSELECT_H

