/**
 * \file imagesourceconstant.h <spatmod/imagesourceconstant.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_CONSTANT definition.
 *
 * \if NODOC
 * $Id: imagesourceconstant.h_v 1.4 2008/04/22 17:18:23 mju Exp $
 *
 * $Log: imagesourceconstant.h_v $
 * Revision 1.4  2008/04/22 17:18:23  mju
 * Override v_getDimensions.
 *
 * Revision 1.3  2006/11/01 23:09:49  mju
 * Chg to queryIterator.
 *
 * Revision 1.2  2006/06/26 17:00:14  mju
 * Remove unneeded fwd declaration.
 *
 * Revision 1.1  2006/02/14 13:50:13  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCECONSTANT_H
#define	INC_SPATMOD_IMAGESOURCECONSTANT_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

class TRANS2D_MAPGEN;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source which supports 'constant' value.
class SOURCE_CONSTANT : public SOURCE {
	public:

		//! Construct with specified dimensions and optional transformation.
		explicit SOURCE_CONSTANT (
			const PIXEL& PixelValue,						//!< Pixel value
			UINT32 TotalColumns,								//!< Total columns
			UINT32 TotalRows,									//!< Total rows
			const TRANS2D_MAPGEN *pTransToCRS = 0		//!< Transformation from device to map CRS
			);

		//! Construct to match specified reference image.
		explicit SOURCE_CONSTANT (
			const PIXEL& PixelValue,						//!< Pixel value
			const BASE& RefImage								//!< Reference image
			);

		// Destructor.
		virtual ~SOURCE_CONSTANT ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_CONSTANT (const SOURCE_CONSTANT&);
		SOURCE_CONSTANT& operator= (const SOURCE_CONSTANT&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCECONSTANT_H

