/**
 * \file imagesourceempty.h <spatmod/imagesourceempty.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_EMPTY definition.
 *
 * \if NODOC
 * $Id: imagesourceempty.h_v 1.3 2009/07/29 15:14:21 mju Exp $
 *
 * $Log: imagesourceempty.h_v $
 * Revision 1.3  2009/07/29 15:14:21  mju
 * Add colorpalette parm to ctors.
 *
 * Revision 1.2  2008/04/22 17:23:02  mju
 * Override v_getDimensions with zoom value.
 *
 * Revision 1.1  2008/04/22 16:38:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEEMPTY_H
#define	INC_SPATMOD_IMAGESOURCEEMPTY_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

class TRANS2D_MAPGEN;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source which supports 'empty' value.
class SOURCE_EMPTY : public SOURCE {
	public:

		//! Construct with specified dimensions and optional transformation.
		explicit SOURCE_EMPTY (
			const PIXEL& PixelProperties,					//!< Pixel value
			UINT32 TotalColumns,								//!< Total columns
			UINT32 TotalRows,									//!< Total rows
			const TRANS2D_MAPGEN *pTransToCRS = 0,		//!< Transformation from device to map CRS
			const PIXELTABLE *pPixelTable = 0			//!< Pixel lookup table for PIXEL::TYPE_Indexed
			);

		//! Construct to match specified reference image.
		explicit SOURCE_EMPTY (
			const PIXEL& PixelProperties,					//!< Pixel value
			const BASE& RefImage,							//!< Reference image
			const PIXELTABLE *pPixelTable = 0			//!< Pixel lookup table for PIXEL::TYPE_Indexed
			);

		// Destructor.
		virtual ~SOURCE_EMPTY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_EMPTY (const SOURCE_EMPTY&);
		SOURCE_EMPTY& operator= (const SOURCE_EMPTY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEEMPTY_H

