/**
 * \file imagesourceplaceholder.h <spatmod/imagesourceplaceholder.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_PLACEHOLDER definition.
 *
 * \if NODOC
 * $Id: imagesourceplaceholder.h_v 1.1 2007/01/22 16:21:05 mju Exp $
 *
 * $Log: imagesourceplaceholder.h_v $
 * Revision 1.1  2007/01/22 16:21:05  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEPLACEHOLDER_H
#define	INC_SPATMOD_IMAGESOURCEPLACEHOLDER_H

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#include <spatmod/imagesource.h>
#endif

class TRANS2D_MAPGEN;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Source to use for "place holder" while waiting for data to become available.
class SOURCE_PLACEHOLDER : public SOURCE {
	public:

		enum STYLE {
			STYLE_Empty,
			STYLE_BorderDash
			};

		//! Construct with specified dimensions and optional transformation.
		explicit SOURCE_PLACEHOLDER (
			const PIXEL& PixelProp,							//!< Pixel properties and value if needed by style
			STYLE Style,										//!< Style
			UINT32 TotalColumns,								//!< Total columns
			UINT32 TotalRows,									//!< Total rows
			const TRANS2D_MAPGEN *pTransToCRS = 0		//!< Transformation from device to map CRS
			);

		//! Construct to match specified reference image.
		explicit SOURCE_PLACEHOLDER (
			const PIXEL& PixelProp,							//!< Pixel properties and value if needed by style
			STYLE Style,										//!< Style
			const BASE& RefImage								//!< Reference image
			);

		// Destructor.
		virtual ~SOURCE_PLACEHOLDER ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		STYLE m_Style;

		void DoColumnDashes (BUFFER& buffer, INT32 bufcol, INT32 MinRow, INT32 NumRows ) const;
		void DoRowDashes (BUFFER& buffer, INT32 bufrow, INT32 MinColumn, INT32 NumColumns) const;

		// BASE overrides.
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_ComputeSampleStats (STAT_UNIVARIATE *SampleStatsArray, STATSVALIDITY StatsValidity);
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_PLACEHOLDER (const SOURCE_PLACEHOLDER&);
		SOURCE_PLACEHOLDER& operator= (const SOURCE_PLACEHOLDER&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEPLACEHOLDER_H

