/**
 * \file imagetargetnull.h <spatmod/imagetargetnull.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_NULL definition.
 *
 * \if NODOC
 * $Id: imagetargetnull.h_v 1.1 2006/11/14 18:26:55 mju Exp $
 *
 * $Log: imagetargetnull.h_v $
 * Revision 1.1  2006/11/14 18:26:55  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETNULL_H
#define	INC_SPATMOD_IMAGETARGETNULL_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Null image target.
class TARGET_NULL : public TARGET {
	public:

		//! Construct target with specified input stage
		explicit TARGET_NULL (
			STAGE *pInputStage							//!< Stage in pipeline to obtain image data from
			);

		//! Destructor.
		virtual ~TARGET_NULL ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();
		virtual ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);

		// Unimplemented
		TARGET_NULL (const TARGET_NULL&);
		TARGET_NULL& operator= (const TARGET_NULL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETNULL_H

